#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Аудиомодальность
"""
# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Подавление Warning
import warnings

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

from dataclasses import dataclass  # Класс данных

import os  # Взаимодействие с файловой системой
import re  # Регулярные выражения
import subprocess  # Работа с процессами
import numpy as np  # Научные вычисления
import torch  # Машинное обучение от Facebook
import torchvision  # Работа с видео от Facebook
import torchaudio  # Работа с аудио от Facebook
import filetype  # Определение типа файла и типа MIME
import json  # Кодирование и декодирование данных в удобном формате
from PIL import Image  # Считывание изображений
from imgaug import augmenters as iaa  # Базовая аугментация
import random  # Случайные числа
import librosa
import librosa.display
from matplotlib import cm

# Парсинг URL
import urllib.parse
import urllib.error

from IPython.utils import io  # Подавление вывода
from pathlib import Path, PosixPath  # Работа с путями в файловой системе
from datetime import datetime, timedelta  # Работа со временем
from pymediainfo import MediaInfo  # Получение meta данных из медиафайлов

from vosk import Model, KaldiRecognizer, SetLogLevel  # Распознавание речи

# Типы данных
from typing import List, Dict, Union, Optional

from types import FunctionType

# Персональные
from openav.modules.core.exceptions import (
    TypeEncodeVideoError,
    PresetCFREncodeVideoError,
    SRInputTypeError,
    IsNestedCatalogsNotFoundError,
    IsNestedDirectoryVNotFoundError,
    IsNestedDirectoryANotFoundError,
    SamplingRateError,
    WindowSizeSamplesError,
    CropPXError,
    CropPercentsError,
    FlipLRProbabilityError,
    FlipUDProbabilityError,
    BlurError,
    ScaleError,
    RotateError,
    ContrastError,
    MixUpAlphaError,
)
from openav.modules.file_manager.yaml_manager import Yaml  # Класс для работы с YAML

# ######################################################################################################################
# Константы
# ######################################################################################################################
TYPES_ENCODE: List[str] = ["qscale", "crf"]  # Типы кодирования
CRF_VALUE: int = 23  # Качество кодирования (от 0 до 51)
# Скорость кодирования и сжатия
PRESETS_CRF_ENCODE: List[str] = [
    "ultrafast",
    "superfast",
    "veryfast",
    "faster",
    "fast",
    "medium",
    "slow",
    "slower",
    "veryslow",
]
SR_INPUT_TYPES: List[str] = ["audio", "video"]  # Типы файлов для распознавания речи
SAMPLING_RATE_VAD: List[int] = [8000, 16000]  # Частота дискретизации
THRESHOLD_VAD: float = 0.56  # Порог вероятности речи (от 0.0 до 1.0)
MIN_SPEECH_DURATION_MS_VAD: int = 250  # Минимальная длительность речевого фрагмента в миллисекундах
# Минимальная длительность тишины в выборках между отдельными речевыми фрагментами
MIN_SILENCE_DURATION_MS_VAD: int = 50
SAMPLING_RATE_MS: List[int] = [16000, 22050, 44100, 48000]  # Частота дискретизации
PAD_MODE_MS: List[int] = ["constant", "reflect", "replicate", "circular"]  # Управление оступами
DPI: List[int] = [72, 96, 150, 300, 600, 1200]  # DPI
COLOR_GRADIENTS: List[str] = ["viridis", "plasma", "inferno", "magma", "cividis"]
EXT_AUDIO_L: List[str] = ["mov", "mp4", "webm", "wav"]  # Расширения искомых файлов

# Количество выборок в каждом окне
# (512, 1024, 1536 для частоты дискретизации 16000 или 256, 512, 768 для частоты дискретизации 8000)
WINDOW_SIZE_SAMPLES_VAD: Dict[int, List[int]] = {8000: [256, 512, 768], 16000: [512, 1024, 1536]}
SPEECH_PAD_MS: int = 150  # Внутренние отступы для итоговых речевых фрагментов
# Суффиксы каналов аудиофрагментов
FRONT: Dict[str, List[str]] = {"mono": ["_mono"], "stereo": ["_left", "_right"]}
EXT_AUDIO_AUG: str = "png"  # Расширение для сохраняемого аудио
EXT_AUDIO_SPEC: str = "png"  # Расширение для сохраняемой MelSpectrogram
EXT_AUDIO: str = "wav"  # Расширение для сохраняемого аудио
EXT_NPY: str = "npy"  # Расширения для сохранения сырых данных MelSpectrogram
VOSK_SUPPORTED_LANGUAGES: List[str] = ["ru", "en"]  # Поддерживаемые языки (Vosk)
VOSK_SUPPORTED_DICTS: List[str] = ["small", "big"]  # Размеры словарей (Vosk)
VOSK_SPEECH_LEFT_PAD_MS: int = 0  # Внутренний левый отступ для итоговых речевых фрагментов
VOSK_SPEECH_RIGHT_PAD_MS: int = 0  # Внутренний правый отступ для итоговых речевых фрагментов


AUGMENTATION_CROP_PX: List[int] = [0, 1000000]  # Диапазон значений обрезки в пикселях
AUGMENTATION_CROP_PERCENT: List[float] = [0, 1.0]  # Диапазон значений обрезки в процентах
AUGMENTATION_FLIP_LR_PROBABILITY: List[float] = [0, 1.0]  # Диапазон значений вероятности отражения по вертикальной оси
AUGMENTATION_FLIP_UD_PROBABILITY: List[float] = [
    0,
    1.0,
]  # Диапазон значений вероятности отражения по горизонтальной оси
AUGMENTATION_BLUR: List[float] = [0, 3.0]  # Диапазон значений размытия
AUGMENTATION_SCALE_X: List[float] = [0, 10.0]  # Диапазон значений масштабирования по оси X
AUGMENTATION_SCALE_Y: List[float] = [0, 10.0]  # Диапазон значений масштабирования по оси Y
AUGMENTATION_ROTATE: List[int] = [-90, 90]  # Диапазон значений поворота
AUGMENTATION_CONTRAST: List[float] = [-10.0, 10.0]  # Диапазон значений контраста
AUGMENTATION_ALPHA: List[float] = [0, 1.0]  # Диапазон значений параметра
AUGMENTATION_COUNT: List[int] = [0, 10000]  # Количество применений аугментации

# Метрики оценки нейросетевой модели
METRICS_AUDIO: List[str] = [
    "accuracy",
]


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class AudioMessages(Yaml):
    """Класс для сообщений

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._wrong_type_encode: str = self._('Тип кодирования видео должен быть одним из "{}"') + self._em
        self._wrong_preset_crf_encode: str = (
            self._("Скорость кодирования и сжатия видео должна быть " 'одной из "{}"') + self._em
        )
        self._wrong_sr_input_type: str = self._('Тип файлов для распознавания должен быть одним из "{}"') + self._em
        self._wrong_sampling_rate_vad: str = (
            self._('Частота дискретизации речевого сигнала должна быть одной из "{}"') + self._em
        )
        self._wrong_window_size_samples_type: str = (
            self._('Для частоты дискретизации "{}" количество выборок в каждом окне должно быть одним из "{}"')
            + self._em
        )

        self._download_model_from_repo: str = self._('Загрузка VAD модели "{}" из репозитория {}') + self._em

        # Переопределение
        self._automatic_download: str = self._("Загрузка Vosk модели") + ' "{}"' + self._em
        self._automatic_download_progress: str = self._automatic_download + " {}%" + self._em

        self._vosk_model_activation: str = self._("Активация Vosk модели") + ' "{}"' + self._em
        self._sr_not_recognized: str = self._("Речь не найдена") + self._em

        self._wrong_crop_px_aug: str = self._('Значение обрезки в пикселях должно быть в пределах "{}"') + self._em
        self._wrong_crop_percent_aug: str = (
            self._('Значение обрезки в процентах должно быть в пределах "{}"') + self._em
        )
        self._wrong_flip_lr_prob_aug: str = (
            self._('Значение вероятности отражения по вертикальной оси должно быть в пределах "{}"') + self._em
        )
        self._wrong_flip_ud_prob_aug: str = (
            self._('Значение вероятности отражения по горизонтальной оси должно быть в пределах "{}"') + self._em
        )
        self._wrong_blur_aug: str = self._('Значение размытия должно быть в пределах "{}"') + self._em
        self._wrong_scale_aug: str = self._('Значение масштабирования должно быть в пределах "{}"') + self._em
        self._wrong_rotate_aug: str = self._('Значение угла поворота должно быть в пределах "{}"') + self._em
        self._wrong_contrast_aug: str = self._('Значение контрастности должно быть в пределах "{}"') + self._em
        self._wrong_alpha_aug: str = self._('Значение коэффициента для MixUp должно быть в пределах "{}"') + self._em

        self._subfolders_search: str = (
            self._('Поиск вложенных директорий в директории "{}" (глубина вложенности: {})') + self._em
        )
        self._subfolders_not_found: str = self._("В указанной директории вложенные директории не найдены") + self._em

        self._files_av_find: str = (
            self._('Поиск файлов с расширениями "{}" в директории "{}" (глубина вложенности: {})') + self._em
        )

        self._files_analysis: str = self._("Анализ файлов") + self._em

        self.preprocess_audio_files: str = self._("Предобработка речевых аудиоданных") + self._em

        self._url_error: str = self._("Не удалось скачать модель{}") + self._em
        self._url_error_code: str = self._(" (ошибка {})")

        self._vad_true: str = self._("Все файлы успешно проанализированы") + self._em

        self._aug_true: str = self._("Все файлы успешно обработаны") + self._em

        self._preprocess_true: str = self._("Все файлы успешно предобработаны") + self._em


# ######################################################################################################################
# Аудио
# ######################################################################################################################
@dataclass
class Audio(AudioMessages):
    """Класс для обработки аудиомодальности

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._github_repo_vad: str = "snakers4/silero-vad"  # Репозиторий для загрузки VAD
        self._vad_model: str = "silero_vad"  # VAD модель

        # Пути к моделям распознавания речи
        self._vosk_models_url: Dict = {"vosk": "https://alphacephei.com/vosk/models/"}

        # Модели для распознавания речи
        self._vosk_models_for_sr: Dict = {
            "vosk": {
                "languages": VOSK_SUPPORTED_LANGUAGES,  # Поддерживаемые языки
                "dicts": VOSK_SUPPORTED_DICTS,  # Размеры словарей
                # Русский язык
                "ru": {"big": "vosk-model-ru-0.42.zip", "small": "vosk-model-small-ru-0.22.zip"},
                # Английский язык
                "en": {"big": "vosk-model-en-us-0.22.zip", "small": "vosk-model-small-en-us-0.15.zip"},
            },
        }

        self.vosk_language_sr: str = VOSK_SUPPORTED_LANGUAGES[0]  # Язык для распознавания речи (Vosk)
        self.vosk_dict_language_sr: str = VOSK_SUPPORTED_DICTS[1]  # Размер словаря для распознавания речи (Vosk)

        # ----------------------- Только для внутреннего использования внутри класса

        self.__model_vad: Optional[torch.jit._script.RecursiveScriptModule] = None  # VAD модель
        self.__get_speech_ts: Optional[FunctionType] = None  # Временные метки VAD

        self.__len_paths: int = 0  # Количество аудиовизуальных файлов
        self.__curr_path: PosixPath = ""  # Текущий аудиовизуальный файл
        self.__splitted_path: str = ""  # Локальный путь до директории
        self.__i: int = 0  # Счетчик
        self.__local_path: List[Union[List[PosixPath], str]] = ""  # Локальный путь
        self.__aframes: torch.Tensor = torch.empty((), dtype=torch.float32)  # Аудиокадры

        self.__dataset_video_vad: List[str] = []  # Пути до директорий с разделенными видеофрагментами
        self.__dataset_audio_vad: List[str] = []  # Пути до директорий с разделенными аудиофрагментами
        self.__dataset_preprocess_audio: List[str] = []  # Пути до директорий с спектрограммами
        self.__unprocessed_files: List[str] = []  # Пути к файлам на которых VAD не отработал
        self.__not_saved_files: List[str] = []  # Пути к файлам которые не сохранились при обработке VAD

        # Результат дочернего процесс распознавания речи
        self.__subprocess_vosk_sr: Union[List[str], Dict[str, List[str]]] = []

        self.__type_encode: str = ""  # Тип кодирования
        self.__crf_value: int = 0  # Качество кодирования (от 0 до 51)
        self.__presets_crf_encode: str = ""  # Скорость кодирования и сжатия
        self.__sr_input_type: str = ""  # Тип файлов для распознавания речи
        self.__sampling_rate_vad: int = 0  # Частота дискретизации
        self.__threshold_vad: float = 0.0  # Порог вероятности речи
        self.__min_speech_duration_ms_vad: int = 0  # Минимальная длительность речевого фрагмента в миллисекундах
        # Минимальная длительность тишины в выборках между отдельными речевыми фрагментами
        self.__min_silence_duration_ms_vad: int = 0
        self.__window_size_samples_vad: int = 0  # Количество выборок в каждом окне
        self.__speech_pad_ms_vad: int = 0  # Внутренние отступы для итоговых речевых фрагментов

        # Метаданные для видео и аудио
        self.__file_metadata: Dict[
            str,
            Union[int, float],
        ] = {"video_fps": 0.0, "audio_fps": 0}

        self.__front: List[str] = []  # Суффиксы каналов аудиофрагментов
        self.__part_video_path: str = ""  # Путь до видеофрагмента
        self.__part_audio_path: str = ""  # Путь до аудиофрагмента

        self.__curr_ts: str = ""  # Текущее время (TimeStamp)

        self.__freq_sr: int = 16000  # Частота дискретизации

        self.__speech_model: Optional[Model] = None  # Модель распознавания речи
        self.__speech_rec: Optional[KaldiRecognizer] = None  # Активация распознавания речи
        self.__keys_speech_rec: List[str] = ["result", "text"]  # Ключи из результата распознавания речи

        self.__vosk_speech_left_pad_ms: int = 0  # Внутренний левый отступ для итоговых речевых фрагментов
        self.__vosk_speech_right_pad_ms: int = 0  # Внутренний правый отступ для итоговых речевых фрагментов

    # ------------------------------------------------------------------------------------------------------------------
    # Свойства
    # ------------------------------------------------------------------------------------------------------------------

    @property
    def vosk_language_sr(self) -> str:
        """Получение/установка языка для распознавания речи

        Args:
            (str): Язык

        Returns:
            str: Язык
        """

        return self.__language_sr

    @vosk_language_sr.setter
    def vosk_language_sr(self, lang: str):
        """Установка языка для распознавания речи"""

        try:
            # Проверка аргументов
            if type(lang) is not str or (lang in VOSK_SUPPORTED_LANGUAGES) is False:
                raise TypeError
        except TypeError:
            pass
        else:
            self.__language_sr = lang

    @property
    def vosk_dict_language_sr(self) -> str:
        """Получение/установка размера словаря для распознавания речи

        Args:
            (str): Размер словаря

        Returns:
            str: Размер словаря
        """

        return self.__dict_language_sr

    @vosk_dict_language_sr.setter
    def vosk_dict_language_sr(self, dict_size: str):
        """Установка размера словаря для распознавания речи"""

        try:
            # Проверка аргументов
            if type(dict_size) is not str or (dict_size in VOSK_SUPPORTED_DICTS) is False:
                raise TypeError
        except TypeError:
            pass
        else:
            self.__dict_language_sr = dict_size

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (приватные)s
    # ------------------------------------------------------------------------------------------------------------------

    # Детальная информация о текущем процессе распознавания речи (Vosk)
    @staticmethod
    def __speech_rec_result(
        keys: List[str], speech_rec_res: Dict[str, Union[List[Dict[str, Union[float, str]]], str]]
    ) -> List[Union[str, float]]:
        """Детальная информация о текущем процессе распознавания речи (Vosk)

        Args:
            keys (List[str]): Ключи из результата распознавания
            speech_rec_res (Dict[str, Union[List[Dict[str, Union[float, str]]], str]]): Текущий результат

        Returns:
            List[Union[str, float]]: Список из текстового представления речи, начала и конца речи
        """

        # Детальная информация распознавания
        if keys[0] in speech_rec_res.keys():
            start = speech_rec_res[keys[0]][0]["start"]  # Начало речи

            if len(speech_rec_res[keys[0]]) == 1:
                idx = 0  # Индекс
            else:
                idx = -1  # Индекс

            end = speech_rec_res[keys[0]][idx]["end"]  # Конец речи
            curr_text = speech_rec_res[keys[1]]  # Распознанный текст

            return [curr_text, round(start, 2), round(end, 2)]  # Текущий результат

        return []

    def __subprocess_vosk_sr_video(self, out: bool) -> List[str]:
        """Дочерний процесс распознавания речи (Vosk) - видео

        Args:
            out (bool) Отображение

        Returns:
            List[str]: Список с текстовыми представлениями речи, начала и конца речи
        """

        try:
            # https://trac.ffmpeg.org/wiki/audio%20types
            # Выполнение в новом процессе
            with subprocess.Popen(
                ["ffmpeg", "-loglevel", "quiet", "-i", self.__curr_path]
                + ["-ar", str(self.__freq_sr), "-ac", str(1), "-f", "s16le", "-"],
                stdout=subprocess.PIPE,
            ) as process:
                results_recognized = []  # Результаты распознавания

                while True:
                    data = process.stdout.read(4000)
                    if len(data) == 0:
                        break

                    curr_res = []  # Текущий результат

                    # Распознанная речь
                    if self.__speech_rec.AcceptWaveform(data):
                        speech_rec_res = json.loads(self.__speech_rec.Result())  # Текущий результат

                        # Детальная информация распознавания
                        curr_res = self.__speech_rec_result(self.__keys_speech_rec, speech_rec_res)
                    else:
                        self.__speech_rec.PartialResult()

                    if len(curr_res) == 3:
                        results_recognized.append(curr_res)

                speech_rec_fin_res = json.loads(self.__speech_rec.FinalResult())  # Итоговый результат распознавания
                # Детальная информация распознавания
                speech_rec_fin_res = self.__speech_rec_result(self.__keys_speech_rec, speech_rec_fin_res)

                # Результат распознавания
                if len(speech_rec_fin_res) == 3:
                    results_recognized.append(speech_rec_fin_res)

                if len(results_recognized) == 0:
                    self.message_error(self._sr_not_recognized, out=out)
                    return []

                return results_recognized
        except OSError:
            self.message_error(self._sr_not_recognized, out=out)
            return []
        except Exception:
            self.message_error(self._unknown_err, out=out)
            return []

    # Дочерний процесс распознавания речи (Vosk) - аудио
    def __subprocess_vosk_sr_audio(self, out: bool) -> Dict[str, List[str]]:
        """Дочерний процесс распознавания речи (Vosk)  - аудио

        Args:
            out (bool) Отображение

        Returns:
            Dict[str, List[str]]: Словарь со вложенными списками из текстового представления речи, начала и конца речи
        """

        # Количество каналов в аудиодорожке
        channels_audio = MediaInfo.parse(self.__curr_path).to_data()["tracks"][1]["channel_s"]

        # Количество каналов больше 2
        if channels_audio > 2:
            self.__unprocessed_files.append(self.__curr_path)
            return {}

        map_channels = {"Mono": "0.0.0"}  # Извлечение моно
        if channels_audio == 2:
            map_channels = {"Left": "0.0.0", "Right": "0.0.1"}  # Стерео

        try:
            results_recognized = {}  # Результаты распознавания

            # Проход по всем каналам
            for front, channel in map_channels.items():
                results_recognized[front] = []  # Словарь для результатов определенного канала
                # https://trac.ffmpeg.org/wiki/audio%20types
                # Выполнение в новом процессе
                with subprocess.Popen(
                    ["ffmpeg", "-loglevel", "quiet", "-i", self.__curr_path]
                    + [
                        "-ar",
                        str(self.__freq_sr),
                        "-map_channel",
                        channel,
                        "-acodec",
                        "pcm_s16le",
                        "-ac",
                        str(1),
                        "-f",
                        "s16le",
                        "-",
                    ],
                    stdout=subprocess.PIPE,
                ) as process:
                    while True:
                        data = process.stdout.read(4000)
                        if len(data) == 0:
                            break

                        curr_res = []  # Текущий результат

                        # Распознанная речь
                        if self.__speech_rec.AcceptWaveform(data):
                            speech_rec_res = json.loads(self.__speech_rec.Result())  # Текущий результат

                            # Детальная информация распознавания
                            curr_res = self.__speech_rec_result(self.__keys_speech_rec, speech_rec_res)
                        else:
                            self.__speech_rec.PartialResult()

                        if len(curr_res) == 3:
                            results_recognized[front].append(curr_res)

                    speech_rec_fin_res = json.loads(self.__speech_rec.FinalResult())  # Итоговый результат распознавания
                    # Детальная информация распознавания
                    speech_rec_fin_res = self.__speech_rec_result(self.__keys_speech_rec, speech_rec_fin_res)

                    # Результат распознавания
                    if len(speech_rec_fin_res) == 3:
                        results_recognized[front].append(speech_rec_fin_res)

            if bool([l for l in results_recognized.values() if l != []]) is False:
                self.message_error(self._sr_not_recognized, out=out)
                return {}

            return results_recognized
        except OSError:
            self.message_error(self._sr_not_recognized, out=out)
            return {}
        except Exception:
            self.message_error(self._unknown_err, out=out)
            return {}

    def __audio_analysis(self) -> bool:
        """Анализ аудиодорожки (VAD)

        Returns:
            bool: **True** если анализ аудиодорожки произведен, в обратном случае **False**
        """

        # Количество каналов в аудиодорожке
        channels_audio = self.__aframes.shape[0]
        if channels_audio > 2:
            self.__unprocessed_files.append(self.__curr_path)
            return False

        if channels_audio == 1:
            self.__front = FRONT["mono"]  # Моно канал
        elif channels_audio == 2:
            self.__front = FRONT["stereo"]  # Стерео канал

        # Тип файла
        kind = filetype.guess(self.__curr_path)

        # Текущее время (TimeStamp)
        # см. datetime.fromtimestamp()
        self.__curr_ts = str(datetime.now().timestamp()).replace(".", "_")

        # Проход по всем каналам
        for channel in range(0, channels_audio):
            try:
                # Получение временных меток
                speech_timestamps = self.__get_speech_ts(
                    audio=self.__aframes[channel],
                    model=self.__model_vad,
                    threshold=self.__threshold_vad,
                    sampling_rate=self.__sampling_rate_vad,
                    min_speech_duration_ms=self.__min_speech_duration_ms_vad,
                    max_speech_duration_s=float("inf"),
                    min_silence_duration_ms=self.__min_silence_duration_ms_vad,
                    window_size_samples=self.__window_size_samples_vad,
                    speech_pad_ms=self.__speech_pad_ms_vad,
                    return_seconds=False,
                    visualize_probs=False,
                    progress_tracking_callback=None,
                )
            except Exception:
                self.__unprocessed_files.append(self.__curr_path)
                return False
            else:

                def join_path(dir_va):
                    return os.path.join(self.path_to_dataset_vad, dir_va, self.__splitted_path)

                # Временные метки найдены
                if len(speech_timestamps) > 0:
                    try:
                        # Видео
                        if kind.mime.startswith("video/") is True:
                            if join_path(self.dir_va_names[0]) not in self.__dataset_video_vad:
                                # Директория с разделенными видеофрагментами
                                self.__dataset_video_vad.append(join_path(self.dir_va_names[0]))
                                if not os.path.exists(self.__dataset_video_vad[-1]):
                                    # Директория не создана
                                    if self.create_folder(self.__dataset_video_vad[-1], out=False) is False:
                                        raise IsNestedDirectoryVNotFoundError
                        # Аудио
                        if join_path(self.dir_va_names[1]) not in self.__dataset_audio_vad:
                            # Директория с разделенными аудиофрагментами
                            self.__dataset_audio_vad.append(join_path(self.dir_va_names[1]))
                            if not os.path.exists(self.__dataset_audio_vad[-1]):
                                # Директория не создана
                                if self.create_folder(self.__dataset_audio_vad[-1], out=False) is False:
                                    raise IsNestedDirectoryANotFoundError
                    except (IsNestedDirectoryVNotFoundError, IsNestedDirectoryANotFoundError):
                        self.__unprocessed_files.append(self.__curr_path)
                        return False
                    except Exception:
                        self.__unprocessed_files.append(self.__curr_path)
                        return False

                # Проход по всем найденным меткам
                for cnt, curr_timestamps in enumerate(speech_timestamps):
                    # Начальное время
                    start_time = timedelta(seconds=curr_timestamps["start"] / self.__file_metadata["audio_fps"])
                    # Конечное время
                    end_time = timedelta(seconds=curr_timestamps["end"] / self.__file_metadata["audio_fps"])

                    diff_time = end_time - start_time  # Разница между начальным и конечным временем

                    # Путь до аудиофрагмента
                    self.__part_audio_path = os.path.join(
                        self.__dataset_audio_vad[-1],
                        Path(self.__curr_path).stem
                        + "_"
                        + str(cnt)
                        + self.__front[channel]
                        + "_"
                        + self.__curr_ts
                        + "."
                        + EXT_AUDIO,
                    )

                    # Видео
                    if kind.mime.startswith("video/") is True:
                        # Путь до видеофрагмента
                        self.__part_video_path = os.path.join(
                            self.__dataset_video_vad[-1],
                            Path(self.__curr_path).stem
                            + "_"
                            + str(cnt)
                            + "_"
                            + self.__curr_ts
                            + Path(self.__curr_path).suffix.lower(),
                        )

                    def not_saved_files():
                        return self.__not_saved_files.append([self.__curr_path, start_time, end_time])

                    call_audio, call_video = 0, 0

                    try:
                        # Видео
                        if kind.mime.startswith("video/") is True:
                            if channel == 0:
                                # Варианты кодирования
                                if self.__type_encode == TYPES_ENCODE[0]:
                                    # https://trac.ffmpeg.org/wiki/Encode/MPEG-4
                                    ff_v = 'ffmpeg -loglevel quiet -ss {} -i "{}" -{} 0 -to {} "{}"'.format(
                                        start_time,
                                        self.__curr_path,
                                        self.__type_encode,
                                        diff_time,
                                        self.__part_video_path,
                                    )
                                elif self.__type_encode == TYPES_ENCODE[1]:
                                    # https://trac.ffmpeg.org/wiki/Encode/H.264
                                    ff_v = 'ffmpeg -loglevel quiet -ss {} -i "{}" -{} {} -preset {} -to {} "{}"'.format(
                                        start_time,
                                        self.__curr_path,
                                        self.__type_encode,
                                        self.__crf_value,
                                        self.__presets_crf_encode,
                                        diff_time,
                                        self.__part_video_path,
                                    )
                            if channels_audio == 1:  # Моно канал
                                ff_a = 'ffmpeg -loglevel quiet -i "{}" -vn -c:a pcm_f32le ' '-ss {} -to {} "{}"'.format(
                                    self.__curr_path, start_time, end_time, self.__part_audio_path
                                )
                            elif channels_audio == 2:  # Стерео канал
                                ff_a = (
                                    'ffmpeg -loglevel quiet -i "{}" -vn -c:a pcm_f32le -map_channel 0.1.{} -ss {} '
                                    '-to {} "{}"'.format(
                                        self.__curr_path, channel, start_time, end_time, self.__part_audio_path
                                    )
                                )

                        # Аудио
                        if kind.mime.startswith("audio/") is True:
                            if channels_audio == 1:  # Моно канал
                                ff_a = 'ffmpeg -loglevel quiet -i "{}" -ss {} -to {} -c copy "{}"'.format(
                                    self.__curr_path, start_time, end_time, self.__part_audio_path
                                )
                            elif channels_audio == 2:  # Стерео канал
                                ff_a = 'ffmpeg -loglevel quiet -i "{}" -map_channel 0.0.{} -ss {} -to {} "{}"'.format(
                                    self.__curr_path, channel, start_time, end_time, self.__part_audio_path
                                )
                    except IndexError:
                        not_saved_files()
                    except Exception:
                        not_saved_files()
                    else:
                        # Видео
                        if kind.mime.startswith("video/") is True and channel == 0:
                            call_video = subprocess.call(ff_v, shell=True)
                        # Аудио
                        call_audio = subprocess.call(ff_a, shell=True)

                        try:
                            if call_audio == 1 or call_video == 1:
                                raise OSError
                        except OSError:
                            not_saved_files()
                        except Exception:
                            not_saved_files()
                        else:
                            try:
                                # Видео
                                if kind.mime.startswith("video/") is True:
                                    # Чтение файла
                                    _, _, _ = torchvision.io.read_video(self.__part_video_path)
                                _, _ = torchaudio.load(self.__part_audio_path)
                            except Exception:
                                not_saved_files()
        return True

    def __audio_analysis_vosk_sr(self) -> bool:
        """Анализ аудиодорожки (Vosk)

        Returns:
            bool: **True** если анализ аудиодорожки произведен, в обратном случае **False**
        """

        # Тип файла
        kind = filetype.guess(self.__curr_path)

        # Видео
        if kind.mime.startswith("video/") is True:
            track = 2
        # Аудио
        if kind.mime.startswith("audio/") is True:
            track = 1

        # Количество каналов в аудиодорожке
        channels_audio = MediaInfo.parse(self.__curr_path).to_data()["tracks"][track]["channel_s"]
        if channels_audio > 2:
            self.__unprocessed_files.append(self.__curr_path)
            return False

        if channels_audio == 1:
            self.__front = FRONT["mono"]  # Моно канал
        elif channels_audio == 2:
            self.__front = FRONT["stereo"]  # Стерео канал

        # Текущее время (TimeStamp)
        # см. datetime.fromtimestamp()
        self.__curr_ts = str(datetime.now().timestamp()).replace(".", "_")

        def join_path(dir_va):
            return os.path.join(self.path_to_dataset_vosk_sr, dir_va, self.__splitted_path)

        # Временные метки найдены
        if len(self.__subprocess_vosk_sr) > 0:
            try:
                # Видео
                if kind.mime.startswith("video/") is True:
                    if join_path(self.dir_va_names[0]) not in self.__dataset_video_vad:
                        # Директория с разделенными видеофрагментами
                        self.__dataset_video_vad.append(join_path(self.dir_va_names[0]))
                        if not os.path.exists(self.__dataset_video_vad[-1]):
                            # Директория не создана
                            if self.create_folder(self.__dataset_video_vad[-1], out=False) is False:
                                raise IsNestedDirectoryVNotFoundError
                # Аудио
                if join_path(self.dir_va_names[1]) not in self.__dataset_audio_vad:
                    # Директория с разделенными аудиофрагментами
                    self.__dataset_audio_vad.append(join_path(self.dir_va_names[1]))
                    if not os.path.exists(self.__dataset_audio_vad[-1]):
                        # Директория не создана
                        if self.create_folder(self.__dataset_audio_vad[-1], out=False) is False:
                            raise IsNestedDirectoryANotFoundError
            except (IsNestedDirectoryVNotFoundError, IsNestedDirectoryANotFoundError):
                self.__unprocessed_files.append(self.__curr_path)
                return False
            except Exception:
                self.__unprocessed_files.append(self.__curr_path)
                return False

        # Проход по всем каналам
        for channel in range(0, channels_audio):
            # Распознавание речи по видео
            if type(self.__subprocess_vosk_sr) is list:
                # Проход по всем найденным меткам
                for cnt, curr_timestamps in enumerate(self.__subprocess_vosk_sr):
                    res_vosk_sr = curr_timestamps[0].lower()

                    if res_vosk_sr == "":
                        continue  # Речь не найдена

                    # Начальное время
                    start_time = timedelta(seconds=curr_timestamps[1]) - timedelta(
                        milliseconds=self.__speech_left_pad_ms
                    )
                    # Конечное время
                    end_time = timedelta(seconds=curr_timestamps[2]) + timedelta(
                        milliseconds=self.__speech_right_pad_ms
                    )

                    if start_time < timedelta(seconds=0) is True:
                        start_time = timedelta(seconds=0)

                    diff_time = end_time - start_time  # Разница между начальным и конечным временем

                    # Путь до аудиофрагмента
                    self.__part_audio_path = os.path.join(
                        self.__dataset_audio_vad[-1],
                        Path(self.__curr_path).stem
                        + "_"
                        + str(cnt)
                        + self.__front[channel]
                        + "_"
                        + self.__curr_ts
                        + "."
                        + EXT_AUDIO,
                    )

                    # Видео
                    if kind.mime.startswith("video/") is True and channel == 0:
                        # Путь до видеофрагмента
                        self.__part_video_path = os.path.join(
                            self.__dataset_video_vad[-1],
                            Path(self.__curr_path).stem
                            + "_"
                            + str(cnt)
                            + "_"
                            + self.__curr_ts
                            + Path(self.__curr_path).suffix.lower(),
                        )

                    def not_saved_files():
                        return self.__not_saved_files.append([self.__curr_path, start_time, end_time])

                    call_audio, call_video = 0, 0

                    try:
                        # Видео
                        if kind.mime.startswith("video/") is True:
                            if channel == 0:
                                # Варианты кодирования
                                if self.__type_encode == TYPES_ENCODE[0]:
                                    # https://trac.ffmpeg.org/wiki/Encode/MPEG-4
                                    ff_v = 'ffmpeg -loglevel quiet -ss {} -i "{}" -{} 0 -to {} "{}"'.format(
                                        start_time,
                                        self.__curr_path,
                                        self.__type_encode,
                                        diff_time,
                                        self.__part_video_path,
                                    )
                                elif self.__type_encode == TYPES_ENCODE[1]:
                                    # https://trac.ffmpeg.org/wiki/Encode/H.264
                                    ff_v = 'ffmpeg -loglevel quiet -ss {} -i "{}" -{} {} -preset {} -to {} "{}"'.format(
                                        start_time,
                                        self.__curr_path,
                                        self.__type_encode,
                                        self.__crf_value,
                                        self.__presets_crf_encode,
                                        diff_time,
                                        self.__part_video_path,
                                    )
                            if channels_audio == 1:  # Моно канал
                                ff_a = (
                                    # 'ffmpeg -loglevel quiet -i "{}" -vn -codec:a copy '
                                    'ffmpeg -loglevel quiet -i "{}" -vn -codec:a pcm_f32le '
                                    '-ac {} -ss {} -to {} "{}"'.format(
                                        self.__curr_path, channels_audio, start_time, end_time, self.__part_audio_path
                                    )
                                )
                            elif channels_audio == 2:  # Стерео канал
                                ff_a = (
                                    'ffmpeg -loglevel quiet -i "{}" -vn -codec:a pcm_f32le -map_channel 0.1.{} -ss {} '
                                    '-to {} "{}"'.format(
                                        self.__curr_path, channel, start_time, end_time, self.__part_audio_path
                                    )
                                )

                        # Аудио
                        if kind.mime.startswith("audio/") is True:
                            if channels_audio == 1:  # Моно канал
                                ff_a = 'ffmpeg -loglevel quiet -i "{}" -ss {} -to {} -c copy "{}"'.format(
                                    self.__curr_path, start_time, end_time, self.__part_audio_path
                                )
                            elif channels_audio == 2:  # Стерео канал
                                ff_a = 'ffmpeg -loglevel quiet -i "{}" -map_channel 0.0.{} -ss {} -to {} "{}"'.format(
                                    self.__curr_path, channel, start_time, end_time, self.__part_audio_path
                                )
                    except IndexError:
                        not_saved_files()
                    except Exception:
                        not_saved_files()
                    else:
                        # Видео
                        if kind.mime.startswith("video/") is True and channel == 0:
                            call_video = subprocess.call(ff_v, shell=True)
                        # Аудио
                        call_audio = subprocess.call(ff_a, shell=True)

                        try:
                            if call_audio == 1 or call_video == 1:
                                raise OSError
                        except OSError:
                            not_saved_files()
                        except Exception:
                            not_saved_files()
                        else:
                            try:
                                # Видео
                                if kind.mime.startswith("video/") is True:
                                    # Чтение файла
                                    _, _, _ = torchvision.io.read_video(self.__part_video_path)
                                _, _ = torchaudio.load(self.__part_audio_path)
                            except Exception:
                                not_saved_files()

        # Распознавание речи по аудио
        if type(self.__subprocess_vosk_sr) is dict:
            # Проход по всем найденным меткам
            for key, val in enumerate(self.__subprocess_vosk_sr.items()):
                # Проход по всем найденным меткам
                for cnt, curr_timestamps in enumerate(val[1]):
                    res_vosk_sr = curr_timestamps[0].lower()

                    if res_vosk_sr == "":
                        continue  # Речь не найдена

                    start_time = timedelta(seconds=curr_timestamps[1])  # Начальное время
                    end_time = timedelta(seconds=curr_timestamps[2])  # Конечное время

                    diff_time = end_time - start_time  # Разница между начальным и конечным временем

                    # Путь до аудиофрагмента
                    self.__part_audio_path = os.path.join(
                        self.__dataset_audio_vad[-1],
                        Path(self.__curr_path).stem
                        + "_"
                        + str(cnt)
                        + self.__front[key]
                        + "_"
                        + self.__curr_ts
                        + "."
                        + EXT_AUDIO,
                    )

                    def not_saved_files():
                        return self.__not_saved_files.append([self.__curr_path, start_time, end_time])

                    call_audio = 0

                    try:
                        # Аудио
                        if kind.mime.startswith("audio/") is True:
                            if channels_audio == 1:  # Моно канал
                                ff_a = 'ffmpeg -loglevel quiet -i "{}" -ss {} -to {} -c copy "{}"'.format(
                                    self.__curr_path, start_time, end_time, self.__part_audio_path
                                )
                            elif channels_audio == 2:  # Стерео канал
                                ff_a = 'ffmpeg -loglevel quiet -i "{}" -map_channel 0.0.{} -ss {} -to {} "{}"'.format(
                                    self.__curr_path, key, start_time, end_time, self.__part_audio_path
                                )
                    except IndexError:
                        not_saved_files()
                    except Exception:
                        not_saved_files()
                    else:
                        # Аудио
                        call_audio = subprocess.call(ff_a, shell=True)

                        try:
                            if call_audio == 1:
                                raise OSError
                        except OSError:
                            not_saved_files()
                        except Exception:
                            not_saved_files()
                        else:
                            try:
                                _, _ = torchaudio.load(self.__part_audio_path)
                            except Exception:
                                not_saved_files()

        return True

    def __augmentation_check_settings(
        self,
        crop_px_min: int,
        crop_px_max: int,
        crop_percent_min: float,
        crop_percent_max: float,
        flip_lr_probability: float,
        flip_ud_probability: float,
        blur_min: float,
        blur_max: float,
        scale_x_min: float,
        scale_x_max: float,
        scale_y_min: float,
        scale_y_max: float,
        rotate_min: int,
        rotate_max: int,
        contrast_min: float,
        contrast_max: float,
        alpha: float,
        count: int,
        out: bool,
    ) -> bool:
        try:
            # Проверка настроек
            if (AUGMENTATION_CROP_PX[0] <= crop_px_min <= crop_px_max <= AUGMENTATION_CROP_PX[1]) is False:
                raise CropPXError
            if (
                AUGMENTATION_CROP_PERCENT[0] <= crop_percent_min <= crop_percent_max <= AUGMENTATION_CROP_PERCENT[1]
            ) is False:
                raise CropPercentsError
            if (
                AUGMENTATION_FLIP_LR_PROBABILITY[0] <= flip_lr_probability <= AUGMENTATION_FLIP_LR_PROBABILITY[1]
            ) is False:
                raise FlipLRProbabilityError
            if (
                AUGMENTATION_FLIP_UD_PROBABILITY[0] <= flip_ud_probability <= AUGMENTATION_FLIP_UD_PROBABILITY[1]
            ) is False:
                raise FlipUDProbabilityError
            if (AUGMENTATION_BLUR[0] <= blur_min <= blur_max <= AUGMENTATION_BLUR[1]) is False:
                raise BlurError
            if (AUGMENTATION_SCALE_X[0] <= scale_x_min <= scale_x_max <= AUGMENTATION_SCALE_X[1]) is False:
                raise ScaleError
            if (AUGMENTATION_SCALE_Y[0] <= scale_y_min <= scale_y_max <= AUGMENTATION_SCALE_Y[1]) is False:
                raise ScaleError
            if (
                type(rotate_min) is not int
                or type(rotate_max) is not int
                or (AUGMENTATION_ROTATE[0] <= rotate_min <= rotate_max <= AUGMENTATION_ROTATE[1]) is False
            ):
                raise RotateError
            if (AUGMENTATION_CONTRAST[0] <= contrast_min <= contrast_max <= AUGMENTATION_CONTRAST[1]) is False:
                raise ContrastError
            if (AUGMENTATION_ALPHA[0] <= alpha <= AUGMENTATION_ALPHA[1]) is False:
                raise MixUpAlphaError
        except CropPXError:
            self.message_error(
                self._wrong_crop_px_aug.format(self.message_line(" - ".join(str(x) for x in AUGMENTATION_CROP_PX))),
                out=out,
            )
            return False
        except CropPercentsError:
            self.message_error(
                self._wrong_crop_percent_aug.format(
                    self.message_line(" - ".join(str(x) for x in AUGMENTATION_CROP_PERCENT))
                ),
                out=out,
            )
            return False
        except FlipLRProbabilityError:
            self.message_error(
                self._wrong_flip_lr_prob_aug.format(
                    self.message_line(" - ".join(str(x) for x in AUGMENTATION_FLIP_LR_PROBABILITY))
                ),
                out=out,
            )
            return False
        except FlipUDProbabilityError:
            self.message_error(
                self._wrong_flip_ud_prob_aug.format(
                    self.message_line(" - ".join(str(x) for x in AUGMENTATION_FLIP_UD_PROBABILITY))
                ),
                out=out,
            )
            return False
        except BlurError:
            self.message_error(
                self._wrong_blur_aug.format(self.message_line(" - ".join(str(x) for x in AUGMENTATION_BLUR))),
                out=out,
            )
            return False
        except ScaleError:
            self.message_error(
                self._wrong_scale_aug.format(self.message_line(" - ".join(str(x) for x in AUGMENTATION_SCALE_X))),
                out=out,
            )
            return False
        except RotateError:
            self.message_error(
                self._wrong_rotate_aug.format(self.message_line(" - ".join(str(x) for x in AUGMENTATION_ROTATE))),
                out=out,
            )
            return False
        except ContrastError:
            self.message_error(
                self._wrong_contrast_aug.format(self.message_line(" - ".join(str(x) for x in AUGMENTATION_CONTRAST))),
                out=out,
            )
            return False
        except MixUpAlphaError:
            self.message_error(
                self._wrong_alpha_aug.format(self.message_line(" - ".join(str(x) for x in AUGMENTATION_ALPHA))),
                out=out,
            )
            return False
        else:
            # Только для внутреннего использования внутри класса
            self.__crop_px_min = crop_px_min
            self.__crop_px_max = crop_px_max
            self.__crop_percent_min = crop_percent_min
            self.__crop_percent_max = crop_percent_max
            self.__flip_lr_probability = flip_lr_probability
            self.__flip_ud_probability = flip_ud_probability
            self.__blur_min = blur_min
            self.__blur_max = blur_max
            self.__scale_x_min = scale_x_min
            self.__scale_x_max = scale_x_max
            self.__scale_y_min = scale_y_min
            self.__scale_y_max = scale_y_max
            self.__rotate_min = rotate_min
            self.__rotate_max = rotate_max
            self.__contrast_min = contrast_min
            self.__contrast_max = contrast_max
            self.__alpha = alpha
            self.__count = count
            # Метаданные для видео и аудио
            self.__file_metadata["video_fps"], self.__file_metadata["audio_fps"] = 0.0, 0
        print()
        return True

    def __augmentation_validate_arguments(
        self,
        depth: int,
        crop_px_min: int,
        crop_px_max: int,
        crop_percent_min: float,
        crop_percent_max: float,
        flip_lr_probability: float,
        flip_ud_probability: float,
        blur_min: float,
        blur_max: float,
        scale_x_min: float,
        scale_x_max: float,
        scale_y_min: float,
        scale_y_max: float,
        rotate_min: int,
        rotate_max: int,
        contrast_min: float,
        contrast_max: float,
        alpha: float,
        count: int,
        clear_diraug: bool,
        out: bool,
    ) -> bool:
        try:
            # Проверка аргументов
            if type(depth) is not int or depth < 1 or type(clear_diraug) is not bool or type(out) is not bool:
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.vad.__name__, out=out)
            return False
        else:
            self.__augmentation_check_settings(
                crop_px_min,
                crop_px_max,
                crop_percent_min,
                crop_percent_max,
                flip_lr_probability,
                flip_ud_probability,
                blur_min,
                blur_max,
                scale_x_min,
                scale_x_max,
                scale_y_min,
                scale_y_max,
                rotate_min,
                rotate_max,
                contrast_min,
                contrast_max,
                alpha,
                count,
                out,
            )
        return True

    def __augmentation_parce_directories(self, depth: int, out: bool) -> List[str]:
        # Информационное сообщение
        self.message_info(
            self._subfolders_search.format(
                self.message_line(self.path_to_input_augmentation_directory),
                self.message_line(str(depth)),
            ),
            out=out,
        )

        # Создание директории, где хранятся данные
        if self.create_folder(self.path_to_input_augmentation_directory, out=False) is False:
            return False

        # Получение вложенных директорий, где хранятся данные
        nested_paths = self.get_paths(self.path_to_input_augmentation_directory, depth=depth, out=False)

        # Вложенные директории не найдены
        try:
            if len(nested_paths) == 0:
                raise IsNestedCatalogsNotFoundError
        except IsNestedCatalogsNotFoundError:
            self.message_error(self._subfolders_not_found, space=self._space, out=out)
            return False

        # Информационное сообщение
        self.message_info(
            self._files_av_find.format(
                self.message_line(", ".join(x.replace(".", "") for x in self.ext_search_files)),
                self.message_line(self.path_to_input_augmentation_directory),
                self.message_line(str(depth)),
            ),
            out=out,
        )
        return nested_paths

    def __augmentation_parce_files(self, depth: int, out: bool) -> List[str]:
        nested_paths = self.__augmentation_parce_directories(depth, out)

        paths = []  # Пути до аудиовизуальных файлов

        # Проход по всем вложенным директориям
        for nested_path in nested_paths:
            # Формирование списка с видеофайлами
            for p in Path(nested_path).glob("*"):
                # Добавление текущего пути к видеофайлу в список
                if p.suffix.lower() in self.ext_search_files:
                    paths.append(p.resolve())
        return paths

    def __augmentation_input_directory_is_not_empty(self, paths: List[str], out: bool) -> bool:
        # Директория с набором данных не содержит аудиовизуальных файлов с необходимыми расширениями
        try:
            self.__len_paths = len(paths)  # Количество аудиовизуальных файлов

            if self.__len_paths == 0:
                raise TypeError
        except TypeError:
            self.message_error(self._files_not_found, space=self._space, out=out)
            return False
        except Exception:
            self.message_error(self._unknown_err, space=self._space, out=out)
            return False

    def __augmentation_prepare_directory(self, paths: List[str], clear_diraug: bool, out: bool) -> bool:
        if self.__augmentation_input_directory_is_not_empty(paths, out):
            # Очистка директории для сохранения обработанных аудиовизуальных сигналов
            if clear_diraug is True and os.path.exists(self.path_to_output_augmentation_directory) is True:
                if self.clear_folder(self.path_to_output_augmentation_directory, out=False) is False:
                    return False

    def __augmentation_process_files(self, paths: List[str], clear_diraug: bool, out: bool) -> bool:

        self.__augmentation_prepare_directory(paths, clear_diraug, out)

        self.__unprocessed_files = []  # Пути к файлам на которых аугментация не отработала

        # Информационное сообщение
        self.message_info(self._files_analysis, out=out)

        # Локальный путь
        self.__local_path = lambda lp: os.path.join(
            *Path(lp).parts[-abs((len(Path(lp).parts) - len(Path(self.path_to_input_augmentation_directory).parts))) :]
        )

        # Проход по всем найденным аудиовизуальных файлам
        for i, path in enumerate(paths):
            self.__curr_path = path  # Текущий аудиовизуальный файл
            self.__i = i + 1  # Счетчик

            self.message_progressbar(
                self._curr_progress.format(
                    self.__i,
                    self.__len_paths,
                    round(self.__i * 100 / self.__len_paths, 2),
                    self.message_line(self.__local_path(self.__curr_path)),
                ),
                space=self._space,
                out=out,
            )

            self.__splitted_path = str(
                self.__curr_path.parent.relative_to(Path(self.path_to_input_augmentation_directory))
            ).strip()

            self.__curr_path = str(self.__curr_path)

            try:
                for k in range(self.__count):
                    # Тип файла
                    _ = filetype.guess(self.__curr_path)
                    directory = os.path.join(
                        self.path_to_output_augmentation_directory,
                        Path(self.__curr_path).parent.relative_to(Path(self.path_to_input_augmentation_directory)),
                    )
                    os.makedirs(directory, exist_ok=True)

                    self.__curr_ts = str(datetime.now().timestamp()).replace(".", "_")

                    path = os.path.join(
                        directory,
                        Path(self.__curr_path).stem + "_" + self.__curr_ts + "." + EXT_AUDIO_AUG,
                    )
                    seq = iaa.Sequential(
                        [
                            iaa.Crop(px=(self.__crop_px_min, self.__crop_px_max)),
                            iaa.Crop(percent=(self.__crop_percent_min, self.__crop_percent_max)),
                            iaa.Fliplr(self.__flip_lr_probability),
                            iaa.Flipud(p=self.__flip_ud_probability),
                            iaa.GaussianBlur(sigma=(self.__blur_min, self.__blur_max)),
                            iaa.Affine(
                                scale={
                                    "x": (self.__scale_x_min, self.__scale_x_max),
                                    "y": (self.__scale_y_min, self.__scale_y_max),
                                },
                                rotate=(self.__rotate_min, self.__rotate_max),
                            ),
                            iaa.LinearContrast((self.__contrast_min, self.__contrast_max)),
                        ]
                    )
                    alpha = self.__alpha
                    img = Image.open(self.__curr_path)
                    img_array = np.array(img)
                    img_aug = seq(image=img_array)

                    img2 = Image.open(paths[random.randint(0, len(paths) - 1)])
                    img2_array = np.array(img2)

                    img_res = ((alpha * img_aug) + ((1 - alpha) * img2_array)).astype(np.uint8)
                    img_res = np.array(img_res)

                    Image.fromarray(img_res).save(path, dpi=(600, 600))

            except Exception as err:
                print(err)
                self.__unprocessed_files.append(self.__curr_path)
                self.message_progressbar(close=True, out=out)
                continue

        self.message_progressbar(close=True, out=out)

        # Файлы на которых аугментация не отработала
        unprocessed_files_unique = np.unique(np.array(self.__unprocessed_files)).tolist()

        if len(unprocessed_files_unique) == 0 and len(self.__not_saved_files) == 0:
            self.message_true(self._aug_true, space=self._space, out=out)
            return True

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (защищенные)
    # ------------------------------------------------------------------------------------------------------------------

    # ------------------------------------------------------------------------------------------------------------------
    # Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def vad(
        self,
        depth: int = 1,
        type_encode: str = TYPES_ENCODE[1],
        crf_value: int = CRF_VALUE,
        presets_crf_encode: str = PRESETS_CRF_ENCODE[5],
        sr_input_type: str = SR_INPUT_TYPES[0],
        sampling_rate: int = SAMPLING_RATE_VAD[1],
        threshold: float = THRESHOLD_VAD,
        min_speech_duration_ms: int = MIN_SPEECH_DURATION_MS_VAD,
        min_silence_duration_ms: int = MIN_SILENCE_DURATION_MS_VAD,
        window_size_samples: int = WINDOW_SIZE_SAMPLES_VAD[SAMPLING_RATE_VAD[1]][2],
        speech_pad_ms: int = SPEECH_PAD_MS,
        force_reload: bool = True,
        clear_dirvad: bool = False,
        out: bool = True,
    ) -> bool:
        """VAD (Voice Activity Detector) или (детектирование голосовой активности)

        Args:
            depth (int): Глубина иерархии для получения данных
            type_encode (str): Тип кодирования
            crf_value (int): Качество кодирования (от **0** до **51**)
            presets_crf_encode (str): Скорость кодирования и сжатия
            sr_input_type (str): Тип файлов для распознавания речи
            sampling_rate (int): Частота дискретизации (**8000** или **16000**)
            threshold (float): Порог вероятности речи (от **0.0** до **1.0**)
            min_speech_duration_ms (int): Минимальная длительность речевого фрагмента в миллисекундах
            min_silence_duration_ms (int): Минимальная длительность тишины в выборках между отдельными речевыми
                                           фрагментами
            window_size_samples (int): Количество выборок в каждом окне (**512**, **1024**, **1536** для частоты
                                       дискретизации **16000** или **256**, **512**, **768** для частоты дискретизации
                                       **8000**)
            speech_pad_ms (int): Внутренние отступы для итоговых речевых фрагментов
            force_reload (bool): Принудительная загрузка модели из сети
            clear_dirvad (bool): Очистка директории для сохранения фрагментов аудиовизуального сигнала
            out (bool): Отображение

        Returns:
            bool: **True** если детектирование голосовой активности произведено, в обратном случае **False**

        .. versionadded:: 0.1.0

        .. versionchanged:: 0.1.1

        .. deprecated:: 0.1.0
        """

        try:
            # Проверка аргументов
            if (
                type(depth) is not int
                or depth < 1
                or type(crf_value) is not int
                or not (0 <= crf_value <= 51)
                or type(threshold) is not float
                or not (0.0 <= threshold <= 1.0)
                or type(min_speech_duration_ms) is not int
                or min_speech_duration_ms < 1
                or type(min_silence_duration_ms) is not int
                or min_silence_duration_ms < 1
                or type(speech_pad_ms) is not int
                or speech_pad_ms < 1
                or type(force_reload) is not bool
                or type(clear_dirvad) is not bool
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.vad.__name__, out=out)
            return False
        else:
            try:
                # Проверка настроек
                if type(type_encode) is not str or (type_encode in TYPES_ENCODE) is False:
                    raise TypeEncodeVideoError
                if type(presets_crf_encode) is not str or (presets_crf_encode in PRESETS_CRF_ENCODE) is False:
                    raise PresetCFREncodeVideoError
                if type(sr_input_type) is not str or (sr_input_type in [x.lower() for x in SR_INPUT_TYPES]) is False:
                    raise SRInputTypeError
                if type(sampling_rate) is not int or (sampling_rate in [x for x in SAMPLING_RATE_VAD]) is False:
                    raise SamplingRateError
                if (
                    type(window_size_samples) is not int
                    or (window_size_samples in [x for x in WINDOW_SIZE_SAMPLES_VAD[sampling_rate]]) is False
                ):
                    raise WindowSizeSamplesError
            except TypeEncodeVideoError:
                self.message_error(
                    self._wrong_type_encode.format(
                        self.message_line(", ".join(x.replace(".", "") for x in TYPES_ENCODE))
                    ),
                    out=out,
                )
                return False
            except PresetCFREncodeVideoError:
                self.message_error(
                    self._wrong_preset_crf_encode.format(
                        self.message_line(", ".join(x.replace(".", "") for x in PRESETS_CRF_ENCODE))
                    ),
                    out=out,
                )
                return False
            except SRInputTypeError:
                self.message_error(
                    self._wrong_sr_input_type.format(
                        self.message_line(", ".join(x.replace(".", "") for x in SR_INPUT_TYPES))
                    ),
                    out=out,
                )
                return False
            except SamplingRateError:
                self.message_error(
                    self._wrong_sampling_rate_vad.format(
                        self.message_line(", ".join(str(x) for x in SAMPLING_RATE_VAD))
                    ),
                    out=out,
                )
                return False
            except WindowSizeSamplesError:
                self.message_error(
                    self._wrong_window_size_samples_type.format(
                        self.message_line(str(sampling_rate)),
                        self.message_line(", ".join(str(x) for x in WINDOW_SIZE_SAMPLES_VAD[sampling_rate])),
                    ),
                    out=out,
                )
                return False
            else:
                # Только для внутреннего использования внутри класса
                self.__type_encode = type_encode
                self.__crf_value = crf_value
                self.__presets_crf_encode = presets_crf_encode
                self.__sr_input_type = sr_input_type
                self.__sampling_rate_vad = sampling_rate
                self.__threshold_vad = threshold
                self.__min_speech_duration_ms_vad = min_speech_duration_ms
                self.__min_silence_duration_ms_vad = min_silence_duration_ms
                self.__window_size_samples_vad = window_size_samples
                self.__speech_pad_ms_vad = speech_pad_ms
                # Метаданные для видео и аудио
                self.__file_metadata["video_fps"], self.__file_metadata["audio_fps"] = 0.0, 0

                torch.set_num_threads(1)  # Установка количества потоков для внутриоперационного параллелизма на ЦП

                torch.hub.set_dir(self.path_to_save_models)  # Установка пути к директории для сохранения VAD модели

                # Информационное сообщение
                self.message_info(
                    self._download_model_from_repo.format(
                        self.message_line(self._vad_model),
                        urllib.parse.urljoin("https://github.com/", self._github_repo_vad),
                    ),
                    out=out,
                )

                try:
                    # Подавление вывода
                    with io.capture_output():
                        # Загрузка VAD модели
                        self.__model_vad, utils = torch.hub.load(
                            repo_or_dir=self._github_repo_vad, model=self._vad_model, force_reload=force_reload
                        )
                except FileNotFoundError:
                    self.message_error(
                        self._clear_folder_not_found.format(self.message_line(self.path_to_save_models)),
                        space=self._space,
                        out=out,
                    )
                    return False
                except RuntimeError:
                    self.message_error(self._url_error.format(""), space=self._space, out=out)
                    return False
                except urllib.error.HTTPError as e:
                    self.message_error(
                        self._url_error.format(self._url_error_code.format(self.message_line(str(e.code)))),
                        space=self._space,
                        out=out,
                    )
                    return False
                except urllib.error.URLError:
                    self.message_error(self._url_error.format(""), space=self._space, out=out)
                    return False
                except Exception:
                    self.message_error(self._unknown_err, space=self._space, out=out)
                    return False
                else:
                    self.__get_speech_ts, _, read_audio, _, _ = utils

                    # Информационное сообщение
                    self.message_info(
                        self._subfolders_search.format(
                            self.message_line(self.path_to_dataset),
                            self.message_line(str(depth)),
                        ),
                        out=out,
                    )

                    # Создание директории, где хранятся данные
                    if self.create_folder(self.path_to_dataset, out=False) is False:
                        return False

                    # Получение вложенных директорий, где хранятся данные
                    nested_paths = self.get_paths(self.path_to_dataset, depth=depth, out=False)

                    # Вложенные директории не найдены
                    try:
                        if len(nested_paths) == 0:
                            raise IsNestedCatalogsNotFoundError
                    except IsNestedCatalogsNotFoundError:
                        self.message_error(self._subfolders_not_found, space=self._space, out=out)
                        return False

                    # Информационное сообщение
                    self.message_info(
                        self._files_av_find.format(
                            self.message_line(", ".join(x.replace(".", "") for x in self.ext_search_files)),
                            self.message_line(self.path_to_dataset),
                            self.message_line(str(depth)),
                        ),
                        out=out,
                    )

                    paths = []  # Пути до аудиовизуальных файлов

                    # Проход по всем вложенным директориям
                    for nested_path in nested_paths:
                        # Формирование списка с видеофайлами
                        for p in Path(nested_path).glob("*"):
                            # Добавление текущего пути к видеофайлу в список
                            if p.suffix.lower() in self.ext_search_files:
                                paths.append(p.resolve())

                    # Директория с набором данных не содержит аудиовизуальных файлов с необходимыми расширениями
                    try:
                        self.__len_paths = len(paths)  # Количество аудиовизуальных файлов

                        if self.__len_paths == 0:
                            raise TypeError
                    except TypeError:
                        self.message_error(self._files_not_found, space=self._space, out=out)
                        return False
                    except Exception:
                        self.message_error(self._unknown_err, space=self._space, out=out)
                        return False
                    else:
                        # Очистка директории для сохранения фрагментов аудиовизуального сигнала
                        if clear_dirvad is True and os.path.exists(self.path_to_dataset_vad) is True:
                            if self.clear_folder(self.path_to_dataset_vad, out=False) is False:
                                return False

                        self.__dataset_video_vad = []  # Пути до директорий с разделенными видеофрагментами
                        self.__dataset_audio_vad = []  # Пути до директорий с разделенными аудиофрагментами

                        self.__unprocessed_files = []  # Пути к файлам на которых VAD не отработал

                        # Информационное сообщение
                        self.message_info(self._files_analysis, out=out)

                        # Локальный путь
                        self.__local_path = lambda lp: os.path.join(
                            *Path(lp).parts[-abs((len(Path(lp).parts) - len(Path(self.path_to_dataset).parts))) :]
                        )

                        # Проход по всем найденным аудиовизуальных файлам
                        for i, path in enumerate(paths):
                            self.__curr_path = path  # Текущий аудиовизуальный файл
                            self.__i = i + 1  # Счетчик

                            self.message_progressbar(
                                self._curr_progress.format(
                                    self.__i,
                                    self.__len_paths,
                                    round(self.__i * 100 / self.__len_paths, 2),
                                    self.message_line(self.__local_path(self.__curr_path)),
                                ),
                                space=self._space,
                                out=out,
                            )

                            self.__splitted_path = str(
                                self.__curr_path.parent.relative_to(Path(self.path_to_dataset))
                            ).strip()

                            self.__curr_path = str(self.__curr_path)

                            # Пропуск невалидных значений
                            if not self.__splitted_path or re.search(r"\s", self.__splitted_path) is not None:
                                continue

                            # Тип файла
                            kind = filetype.guess(self.__curr_path)

                            try:
                                # Видео
                                if kind.mime.startswith("video/") is True:
                                    # Чтение файла
                                    _, self.__aframes, self.__file_metadata = torchvision.io.read_video(
                                        self.__curr_path
                                    )

                                # Аудио
                                if kind.mime.startswith("audio/") is True:
                                    (self.__aframes, self.__file_metadata["audio_fps"]) = torchaudio.load(
                                        self.__curr_path
                                    )
                            except Exception:
                                self.__unprocessed_files.append(self.__curr_path)
                                self.message_progressbar(close=True, out=out)
                                continue
                            else:
                                # Аудио
                                if kind.mime.startswith("audio/") is True:
                                    self.__aframes = self.__aframes.to(torch.float32)

                                self.__audio_analysis()  # Анализ аудиодорожки

                        self.message_progressbar(close=True, out=out)

                        # Файлы на которых VAD не отработал
                        unprocessed_files_unique = np.unique(np.array(self.__unprocessed_files)).tolist()

                        if len(unprocessed_files_unique) == 0 and len(self.__not_saved_files) == 0:
                            self.message_true(self._vad_true, space=self._space, out=out)
                            return True

    def vosk(self, new_name: Optional[str] = None, force_reload: bool = True, out: bool = True) -> bool:
        """Загрузка и активация модели Vosk для детектирования голосовой активности и распознавания речи

        Args:
            new_name (str): Имя директории для разархивирования
            force_reload (bool): Принудительная загрузка модели из сети
            out (bool) Отображение

        Returns:
            bool: **True** если модель Vosk загружена и активирована, в обратном случае **False**
        """

        try:
            # Проверка аргументов
            if (
                ((type(new_name) is not str or not new_name) and new_name is not None)
                or type(force_reload) is not bool
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.vosk.__name__, out=out)
            return False
        else:
            name = "vosk"  # Модель для распознавания речи

            SetLogLevel(-1)  # Уровень LOG

            lsr = self.vosk_language_sr  # Язык для распознавания речи
            dlsr = self.vosk_dict_language_sr  # Размер словаря для распознавания речи

            url = urllib.parse.urljoin(self._vosk_models_url[name], self._vosk_models_for_sr[name][lsr][dlsr])

            try:
                # Загрузка файла из URL
                res_download_file_from_url = self.download_file_from_url(url=url, force_reload=force_reload, out=out)
            except Exception:
                self.message_error(self._unknown_err, start=True, space=self._space, out=out)
                return False
            else:
                # Файл загружен
                if res_download_file_from_url == 200:
                    try:
                        # Распаковка архива
                        res_unzip = self.unzip(
                            path_to_zipfile=os.path.join(
                                self.path_to_save_models, self._vosk_models_for_sr[name][lsr][dlsr]
                            ),
                            new_name=new_name,
                            force_reload=force_reload,
                        )
                    except Exception:
                        self.message_error(self._unknown_err, start=True, out=out)
                        return False
                    else:
                        # Файл распакован
                        if res_unzip is True:
                            try:
                                # Информационное сообщение
                                self.message_info(
                                    self._vosk_model_activation.format(
                                        self.message_line(Path(self._vosk_models_for_sr[name][lsr][dlsr]).stem)
                                    ),
                                    start=True,
                                    out=out,
                                )

                                self.__speech_model = Model(str(self._path_to_unzip))  # Активация модели
                                # Активация распознавания речи
                                self.__speech_rec = KaldiRecognizer(self.__speech_model, self.__freq_sr)
                                self.__speech_rec.SetWords(True)  # Данные о начале и конце слова/фразы
                            except Exception:
                                self.message_error(self._unknown_err, out=out)
                            else:
                                return True
                else:
                    return False

    def vosk_sr(
        self,
        depth: int = 1,
        type_encode: str = TYPES_ENCODE[1],
        crf_value: int = CRF_VALUE,
        presets_crf_encode: str = PRESETS_CRF_ENCODE[5],
        new_name: Optional[str] = None,
        speech_left_pad_ms: int = VOSK_SPEECH_LEFT_PAD_MS,
        speech_right_pad_ms: int = VOSK_SPEECH_RIGHT_PAD_MS,
        force_reload: bool = True,
        clear_dirvosk_sr: bool = False,
        out: bool = True,
    ) -> bool:
        """VAD + SR (Voice Activity Detector + Speech Recognition) или (детектирование голосовой активности и
        распознавание речи)

        Args:
            depth (int): Глубина иерархии для получения данных
            type_encode (str): Тип кодирования
            crf_value (int): Качество кодирования (от **0** до **51**)
            presets_crf_encode (str): Скорость кодирования и сжатия
            new_name (str): Имя директории для разархивирования
            speech_left_pad_ms (int): Внутренний левый отступ для итоговых речевых фрагментов
            speech_right_pad_ms (int): Внутренний правый отступ для итоговых речевых фрагментов
            force_reload (bool): Принудительная загрузка модели из сети
            clear_dirvosk_sr (bool): Очистка директории для сохранения фрагментов аудиовизуального сигнала
            out (bool) Отображение

        Returns:
            bool: **True** если детектирование голосовой активности и распознавание речи произведено, в обратном случае
            **False**
        """

        try:
            # Проверка аргументов
            if (
                type(depth) is not int
                or depth < 1
                or type(crf_value) is not int
                or not (0 <= crf_value <= 51)
                or ((type(new_name) is not str or not new_name) and new_name is not None)
                or type(speech_left_pad_ms) is not int
                or speech_left_pad_ms < 0
                or type(speech_right_pad_ms) is not int
                or speech_right_pad_ms < 0
                or type(force_reload) is not bool
                or type(clear_dirvosk_sr) is not bool
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.vosk_sr.__name__, out=out)
            return False
        else:
            try:
                # Проверка настроек
                if type(type_encode) is not str or (type_encode in TYPES_ENCODE) is False:
                    raise TypeEncodeVideoError
                if type(presets_crf_encode) is not str or (presets_crf_encode in PRESETS_CRF_ENCODE) is False:
                    raise PresetCFREncodeVideoError

            except TypeEncodeVideoError:
                self.message_error(
                    self._wrong_type_encode.format(
                        self.message_line(", ".join(x.replace(".", "") for x in TYPES_ENCODE))
                    ),
                    out=out,
                )
                return False
            except PresetCFREncodeVideoError:
                self.message_error(
                    self._wrong_preset_crf_encode.format(
                        self.message_line(", ".join(x.replace(".", "") for x in PRESETS_CRF_ENCODE))
                    ),
                    out=out,
                )
                return False
            else:
                # Только для внутреннего использования внутри класса
                self.__type_encode = type_encode
                self.__crf_value = crf_value
                self.__presets_crf_encode = presets_crf_encode

                self.__speech_left_pad_ms = speech_left_pad_ms
                self.__speech_right_pad_ms = speech_right_pad_ms

                # Информационное сообщение
                self.message_info(
                    self._subfolders_search.format(
                        self.message_line(self.path_to_dataset),
                        self.message_line(str(depth)),
                    ),
                    out=out,
                )

                # Создание директории, где хранятся данные
                if self.create_folder(self.path_to_dataset, out=False) is False:
                    return False

                # Получение вложенных директорий, где хранятся данные
                nested_paths = self.get_paths(self.path_to_dataset, depth=depth, out=False)

                # Вложенные директории не найдены
                try:
                    if len(nested_paths) == 0:
                        raise IsNestedCatalogsNotFoundError
                except IsNestedCatalogsNotFoundError:
                    self.message_error(self._subfolders_not_found, space=self._space, out=out)
                    return False

                # Информационное сообщение
                self.message_info(
                    self._files_av_find.format(
                        self.message_line(", ".join(x.replace(".", "") for x in self.ext_search_files)),
                        self.message_line(self.path_to_dataset),
                        self.message_line(str(depth)),
                    ),
                    out=out,
                )

                paths = []  # Пути до аудиовизуальных файлов

                # Проход по всем вложенным директориям
                for nested_path in nested_paths:
                    # Формирование списка с видеофайлами
                    for p in Path(nested_path).glob("*"):
                        # Добавление текущего пути к видеофайлу в список
                        if p.suffix.lower() in self.ext_search_files:
                            paths.append(p.resolve())

                # Директория с набором данных не содержит аудиовизуальных файлов с необходимыми расширениями
                try:
                    self.__len_paths = len(paths)  # Количество аудиовизуальных файлов

                    if self.__len_paths == 0:
                        raise TypeError
                except TypeError:
                    self.message_error(self._files_not_found, space=self._space, out=out)
                    return False
                except Exception:
                    self.message_error(self._unknown_err, space=self._space, out=out)
                    return False
                else:
                    # Очистка директории для сохранения фрагментов аудиовизуального сигнала
                    if clear_dirvosk_sr is True and os.path.exists(self.path_to_dataset_vosk_sr) is True:
                        if self.clear_folder(self.path_to_dataset_vosk_sr, out=False) is False:
                            return False

                    self.__dataset_video_vad = []  # Пути до директорий с разделенными видеофрагментами
                    self.__dataset_audio_vad = []  # Пути до директорий с разделенными аудиофрагментами

                    self.__unprocessed_files = []  # Пути к файлам на которых VAD не отработал

                    # Загрузка и активация модели Vosk для распознавания речи
                    if self.vosk(new_name=new_name, force_reload=force_reload, out=out) is False:
                        return False

                    # Информационное сообщение
                    self.message_info(self._files_analysis, out=out)

                    # Локальный путь
                    self.__local_path = lambda lp: os.path.join(
                        *Path(lp).parts[-abs((len(Path(lp).parts) - len(Path(self.path_to_dataset).parts))) :]
                    )

                    # Проход по всем найденным аудиовизуальных файлам
                    for i, path in enumerate(paths):
                        self.__curr_path = path  # Текущий аудиовизуальный файл
                        self.__i = i + 1  # Счетчик

                        self.message_progressbar(
                            self._curr_progress.format(
                                self.__i,
                                self.__len_paths,
                                round(self.__i * 100 / self.__len_paths, 2),
                                self.message_line(self.__local_path(self.__curr_path)),
                            ),
                            space=self._space,
                            out=out,
                        )

                        self.__splitted_path = str(
                            self.__curr_path.parent.relative_to(Path(self.path_to_dataset))
                        ).strip()

                        self.__curr_path = str(self.__curr_path)

                        # Пропуск невалидных значений
                        if not self.__splitted_path or re.search(r"\s", self.__splitted_path) is not None:
                            continue

                        # Тип файла
                        kind = filetype.guess(self.__curr_path)

                        try:
                            # Активация распознавания речи
                            self.__speech_rec = KaldiRecognizer(self.__speech_model, self.__freq_sr)
                            self.__speech_rec.SetWords(True)  # Данные о начале и конце слова/фразы
                        except Exception:
                            continue
                        else:
                            try:
                                # Видео
                                if kind.mime.startswith("video/") is True:
                                    #  Дочерний процесс распознавания речи (Vosk) - видео
                                    self.__subprocess_vosk_sr = self.__subprocess_vosk_sr_video(out=False)
                                # Аудио
                                if kind.mime.startswith("audio/") is True:
                                    #  Дочерний процесс распознавания речи (Vosk) - аудио
                                    self.__subprocess_vosk_sr = self.__subprocess_vosk_sr_audio(out=False)
                            except Exception:
                                self.__unprocessed_files.append(self.__curr_path)
                                self.message_progressbar(close=True, out=out)
                                continue
                            else:
                                self.__audio_analysis_vosk_sr()  # Анализ аудиодорожки

                    self.message_progressbar(close=True, out=out)

                    # Файлы на которых VAD не отработал
                    unprocessed_files_unique = np.unique(np.array(self.__unprocessed_files)).tolist()

                    if len(unprocessed_files_unique) == 0 and len(self.__not_saved_files) == 0:
                        self.message_true(self._vad_true, space=self._space, out=out)

    def augmentation(
        self,
        depth: int = 1,
        crop_px_min: int = AUGMENTATION_CROP_PX[0],
        crop_px_max: int = AUGMENTATION_CROP_PX[0],
        crop_percent_min: float = AUGMENTATION_CROP_PERCENT[0],
        crop_percent_max: float = AUGMENTATION_CROP_PERCENT[0],
        flip_lr_probability: float = AUGMENTATION_FLIP_LR_PROBABILITY[0],
        flip_ud_probability: float = AUGMENTATION_FLIP_UD_PROBABILITY[0],
        blur_min: float = AUGMENTATION_BLUR[0],
        blur_max: float = AUGMENTATION_BLUR[0],
        scale_x_min: float = AUGMENTATION_SCALE_X[0],
        scale_x_max: float = AUGMENTATION_SCALE_X[0],
        scale_y_min: float = AUGMENTATION_SCALE_Y[0],
        scale_y_max: float = AUGMENTATION_SCALE_Y[0],
        rotate_min: int = AUGMENTATION_ROTATE[0],
        rotate_max: int = AUGMENTATION_ROTATE[0],
        contrast_min: float = AUGMENTATION_CONTRAST[0],
        contrast_max: float = AUGMENTATION_CONTRAST[0],
        alpha: float = AUGMENTATION_ALPHA[0],
        count: int = 1,
        clear_diraug: bool = False,
        out: bool = True,
    ) -> bool:
        """Аугментация аудиовизуальных сигналов

        Args:
            depth (int): Глубина иерархии для получения данных
            crop_px_min (int): Обрезка в пикселях мин
            crop_px_max (int): Обрезка в пикселях макс
            crop_percent_min (float): Обрезка в процентах мин
            crop_percent_max (float): Обрезка в процентах макс
            flip_lr_probability (float): Вероятность отражения по вертикали
            flip_ud_probability (float): Вероятность отражения по горизонтали
            blur_min (float): Размытие мин
            blur_max (float): Размытие макс
            scale_x_min (float): Масштабирование Х мин
            scale_x_max (float): Масштабирование Х макс
            scale_y_min (float): Масштабирование Y мин
            scale_y_max (float): Масштабирование Y макс
            rotate_min (int): Поворот мин
            rotate_max (int): Поворот макс
            contrast_min (float): Контраст мин
            contrast_max (float): Контраст макс
            alpha (float): Альфа для MixUp
            count (int): Количество применений аугментации
            clear_diraug (bool): Очистка директории для сохранения аугментированных аудиовизуальных сигналов
            out (bool): Отображение

        Returns:
            bool: **True** если аугментация аудиовизуальных сигналов произведено, в обратном случае **False**

        .. versionadded:: 0.1.0

        .. versionchanged:: 0.1.1

        .. deprecated:: 0.1.0
        """
        self.__augmentation_validate_arguments(
            depth,
            crop_px_min,
            crop_px_max,
            crop_percent_min,
            crop_percent_max,
            flip_lr_probability,
            flip_ud_probability,
            blur_min,
            blur_max,
            scale_x_min,
            scale_x_max,
            scale_y_min,
            scale_y_max,
            rotate_min,
            rotate_max,
            contrast_min,
            contrast_max,
            alpha,
            count,
            clear_diraug,
            out,
        )
        paths = self.__augmentation_parce_files(depth, out)
        return self.__augmentation_process_files(paths, clear_diraug, out)

    def preprocess_audio(
        self,
        depth: int = 1,
        sample_rate: int = 16000,
        n_fft: int = 2048,
        hop_length: int = 512,
        n_mels: int = 128,
        power: float = 2.0,
        pad_mode: str = "reflect",
        norm: str = "slaney",
        center: bool = True,
        dpi: int = 1200,
        color_gradients: str = "magma",
        save_raw_data: bool = True,
        clear_dir_audio: bool = False,
        out: bool = True,
    ) -> bool:
        """Предобработка речевых аудиоданных

        Args:
            depth (int): Глубина иерархии для получения данных
            sample_rate (int): Частота дискретизации
            n_fft (int): Размер параметра FFT
            hop_length (int): Длина перехода между окнами STFT
            n_mels (int): Количество фильтроблоков mel
            power (float): Показатель степени магнитудной спектрограммы
            pad_mode (str): Управление оступами
            norm (str): Коэффициенты треугольных mel-фильтров делятся на ширину соответствующих mel-полос
            center (bool): Отступы с обеих сторон относительно центра аудиодорожки
            dpi (int): DPI
            color_gradients (str): Градиент для спектрограммы
            save_raw_data (bool): Сохранение сырых данных мел-спектрограммы в формате .npy
            clear_dir_audio (bool): Очистка директории для сохранения аудиоданных после предобработки
            out (bool) Отображение

        Returns:
            bool: **True** если предобработка речевых аудиоданных произведено, в обратном случае
            **False**
        """

        try:
            # Проверка аргументов
            if (
                type(depth) is not int
                or depth < 1
                or type(sample_rate) is not int
                or (sample_rate in SAMPLING_RATE_MS) is False
                or type(n_fft) is not int
                or not (256 <= n_fft <= 2048)
                or type(hop_length) is not int
                or not (64 <= hop_length <= 512)
                or type(n_mels) is not int
                or not (20 <= n_mels <= 512)
                or type(power) is not float
                or (power in [1.0, 2.0]) is False
                or type(pad_mode) is not str
                or (pad_mode in PAD_MODE_MS) is False
                or type(norm) is not str
                or norm != "slaney"
                or type(center) is not bool
                or type(dpi) is not int
                or (dpi in DPI) is False
                or type(color_gradients) is not str
                or (color_gradients in COLOR_GRADIENTS) is False
                or type(save_raw_data) is not bool
                or type(clear_dir_audio) is not bool
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.preprocess_audio.__name__, out=out)
            return False
        else:
            # Информационное сообщение
            self.message_info(
                self._subfolders_search.format(
                    self.message_line(self.path_to_dataset),
                    self.message_line(str(depth)),
                ),
                out=out,
            )

            # Создание директории, где хранятся данные
            if self.create_folder(self.path_to_dataset, out=False) is False:
                return False

            # Получение вложенных директорий, где хранятся данные
            nested_paths = self.get_paths(self.path_to_dataset, depth=depth, out=False)

            # Вложенные директории не найдены
            try:
                if len(nested_paths) == 0:
                    raise IsNestedCatalogsNotFoundError
            except IsNestedCatalogsNotFoundError:
                self.message_error(self._subfolders_not_found, space=self._space, out=out)
                return False

            # Информационное сообщение
            self.message_info(
                self._files_av_find.format(
                    self.message_line(", ".join(x.replace(".", "") for x in self.ext_search_files)),
                    self.message_line(self.path_to_dataset),
                    self.message_line(str(depth)),
                ),
                out=out,
            )

            paths = []  # Пути до аудиовизуальных файлов

            # Проход по всем вложенным директориям
            for nested_path in nested_paths:
                # Формирование списка с видеофайлами
                for p in Path(nested_path).glob("*"):
                    # Добавление текущего пути к видеофайлу в список
                    if p.suffix.lower() in self.ext_search_files:
                        paths.append(p.resolve())

            # Директория с набором данных не содержит аудиовизуальных файлов с необходимыми расширениями
            try:
                self.__len_paths = len(paths)  # Количество аудиовизуальных файлов

                if self.__len_paths == 0:
                    raise TypeError
            except TypeError:
                self.message_error(self._files_not_found, space=self._space, out=out)
                return False
            except Exception:
                self.message_error(self._unknown_err, space=self._space, out=out)
                return False
            else:
                # Очистка директории для сохранения фрагментов аудиовизуального сигнала
                if clear_dir_audio is True and os.path.exists(self.path_to_dataset_audio) is True:
                    if self.clear_folder(self.path_to_dataset_audio, out=False) is False:
                        return False

                self.__dataset_preprocess_audio = []  # Пути до директорий с спектрограммами

                self.__unprocessed_files = []  # Пути к файлам из которых спектрограммы не сформированы

                # Информационное сообщение
                self.message_info(self.preprocess_audio_files, out=out)

                # Локальный путь
                self.__local_path = lambda lp: os.path.join(
                    *Path(lp).parts[-abs((len(Path(lp).parts) - len(Path(self.path_to_dataset).parts))) :]
                )

                # Проход по всем найденным аудиовизуальных файлам
                for i, path in enumerate(paths):
                    self.__curr_path = path  # Текущий аудиовизуальный файл
                    self.__i = i + 1  # Счетчик

                    self.message_progressbar(
                        self._curr_progress.format(
                            self.__i,
                            self.__len_paths,
                            round(self.__i * 100 / self.__len_paths, 2),
                            self.message_line(self.__local_path(self.__curr_path)),
                        ),
                        space=self._space,
                        out=out,
                    )

                    self.__splitted_path = str(self.__curr_path.parent.relative_to(Path(self.path_to_dataset))).strip()

                    self.__curr_path = str(self.__curr_path)

                    # Пропуск невалидных значений
                    if not self.__splitted_path or re.search(r"\s", self.__splitted_path) is not None:
                        continue

                    # Тип файла
                    kind = filetype.guess(self.__curr_path)

                    try:
                        # Видео или аудио
                        if kind.mime.startswith("video/") is True or kind.mime.startswith("audio/") is True:
                            # Формирование мел-спектрограммы
                            waveform, sample_rate = librosa.load(self.__curr_path, sr=sample_rate)
                            waveform = torch.Tensor(waveform)

                            torchaudio_melspec = torchaudio.transforms.MelSpectrogram(
                                sample_rate=sample_rate,
                                n_fft=n_fft,
                                win_length=None,
                                hop_length=hop_length,
                                center=center,
                                pad_mode=pad_mode,
                                power=power,
                                norm=norm,
                                onesided=True,
                                n_mels=n_mels,
                                f_max=None,
                            )(waveform)

                            # Преобразование мел-спектрограммы в децибелы
                            melspectogram_db_transform = torchaudio.transforms.AmplitudeToDB()
                            melspec_db = melspectogram_db_transform(torchaudio_melspec)

                            # Преобразование мел-спектрограммы в numpy-массив
                            melspec_np = melspec_db.numpy()

                            # Текущее время (TimeStamp)
                            # см. datetime.fromtimestamp()
                            self.__curr_ts = str(datetime.now().timestamp()).replace(".", "_")

                            # Путь до мел-спектрограммы
                            melspec_path = os.path.join(
                                self.path_to_dataset_audio,
                                Path(self.__curr_path).stem + "_" + self.__curr_ts + "." + EXT_AUDIO_SPEC,
                            )

                            if not os.path.exists(self.path_to_dataset_audio):
                                # Директория не создана
                                if self.create_folder(self.path_to_dataset_audio, out=False) is False:
                                    raise FileNotFoundError

                            # Нормализация значений мел-спектрограммы в диапазон [0, 1]
                            melspec_np = (melspec_np - melspec_np.min()) / (melspec_np.max() - melspec_np.min())

                            # Переворот массива по вертикали
                            melspec_np = np.flip(melspec_np, axis=0)

                            # Применение цветовой карты
                            # color_gradients: viridis, plasma, inferno, magma, cividis
                            cmap = cm.get_cmap(color_gradients)
                            melspec_rgb = cmap(melspec_np)[:, :, :3]  # Извлечение только RGB-каналов

                            # Нормализация значений в диапазон [0, 255]
                            melspec_rgb = (melspec_rgb * 255).astype("uint8")

                            # Создание и сохранение изображения с помощью Pillow
                            img = Image.fromarray(melspec_rgb)
                            img.save(melspec_path, dpi=(dpi, dpi))

                            if save_raw_data:
                                # Сохранение сырых данных мел-спектрограммы в формате .npy
                                raw_data_path = melspec_path.replace("." + EXT_AUDIO_SPEC, "." + EXT_NPY)
                                np.save(raw_data_path, melspec_np)
                    except Exception:
                        self.__unprocessed_files.append(self.__curr_path)
                        self.message_progressbar(close=True, out=out)
                        continue

                self.message_progressbar(close=True, out=out)

                # Файлы на которых предварительная обработка не отработала
                unprocessed_files_unique = np.unique(np.array(self.__unprocessed_files)).tolist()

                if len(unprocessed_files_unique) == 0 and len(self.__not_saved_files) == 0:
                    self.message_true(self._preprocess_true, space=self._space, out=out)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Работа с файлами
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Подавление Warning
import warnings

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

import os  # Работа с файловой системой
import re  # Регулярные выражения
import shutil  # Набор функций высокого уровня для обработки файлов, групп файлов, и папок
from pathlib import Path  # Работа с путями в файловой системе

from dataclasses import dataclass  # Класс данных

from typing import List, Optional, Iterable  # Типы данных

# Персональные
from openav.modules.file_manager.unzip import Unzip  # Обработка архивов


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class FileManagerMessages(Unzip):
    """Класс для сообщений

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._folder_not_found: str = self._('Директория "{}" не найдена') + self._em
        self._file_name: str = self._("Необходимо указать название файла с расширением") + " {}" + self._em
        self._dir_found: str = self._("Вместо файла передана директория") + self._em
        self._file_find_hide: str = self._('Поиск "{}" файла') + self._em
        self._file_find: str = self._file_find_hide
        self._wrong_extension: str = self._("Расширение файла должно быть") + " {}" + self._em
        self._file_not_found_create: str = self._('Файл "{}" не найден, но был создан') + self._em
        self._file_not_found: str = self._('Файл "{}" не найден') + self._em
        self._dir_name: str = self._("Необходимо указать название директории") + self._em
        self._files_find: str = self._('Поиск файлов с расширениями "{}" в директории "{}"') + self._em
        self._files_not_found: str = self._("В указанной директории необходимые файлы не найдены") + self._em
        self._create_folder: str = self._('Создание директории "{}"') + self._em
        self._folder_not_create: str = self._('Директория "{}" не создана') + self._em
        self._clear_folder: str = self._('Очистка директории "{}"') + self._em
        self._clear_folder_not_found: str = self._('Директория "{}" не найдена') + self._em


# ######################################################################################################################
# Работа с файлами
# ######################################################################################################################
@dataclass
class FileManager(FileManagerMessages):
    """Класс для работы с файлами

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

    # ------------------------------------------------------------------------------------------------------------------
    #  Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    @staticmethod
    def re_inv_chars(path: str) -> str:
        """Удаление недопустимых символов из пути

        Args:
            path (str): Путь

        Returns:
            str: Путь
        """

        return re.sub('[\\/:"*?<>|]+', "", path)

    def get_paths(self, path: Iterable, depth: int = 1, out: bool = True) -> List[Optional[str]]:
        """Получение поддиректорий

        Args:
            path (Iterable): Путь к директории
            depth (int): Глубина иерархии для извлечения поддиректорий
            out (bool): Отображение

        Returns:
            List[Optional[str]]: Список с поддиректориями
        """

        try:
            # Проверка аргументов
            if (
                not isinstance(path, Iterable)
                or not path
                or type(depth) is not int
                or depth < 1
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.get_paths.__name__, out=out)
            return []
        else:
            if type(path) is not list:
                path = [path]

            new_path = []  # Список с директориями

            # Проход по всем директориям
            for curr_path in path:
                try:
                    scandir = os.scandir(os.path.normpath(str(curr_path)))
                except FileNotFoundError:
                    self.message_error(self._folder_not_found.format(self.message_line(str(curr_path))), out=out)
                    return []
                except Exception as e:
                    self.message_error(self._unknown_err, out=out)
                    print(e)
                    return []
                else:
                    for f in scandir:
                        if f.is_dir() and not f.name.startswith("."):
                            ignore = False  # По умолчанию не игнорировать директорию
                            if depth == 1:
                                for curr_dir in self.ignore_dirs:
                                    if type(curr_dir) is not str:
                                        continue
                                    # Игнорировать директорию
                                    if re.search("^" + curr_dir, f.name) is not None:
                                        ignore = True

                            if ignore is False:
                                new_path.append(f.path)

            # Рекурсивный переход на следующий уровень иерархии
            if depth != 1 and len(new_path) > 0:
                return self.get_paths(new_path, depth - 1)

            return new_path  # Список с директориями

    def search_file(self, path_to_file: str, ext: str, create: bool = False, out: bool = True) -> bool:
        """Поиск файла

        Args:
            path_to_file (str): Путь к файлу
            ext (str): Расширение файла
            create (bool): Создание файла в случае его отсутствия
            out (bool): Печатать процесс выполнения

        Returns:
            bool: **True** если файл найден, в обратном случае **False**
        """

        # Проверка аргументов
        if (
            type(path_to_file) is not str
            or type(ext) is not str
            or not ext
            or type(create) is not bool
            or type(out) is not bool
        ):
            self.inv_args(__class__.__name__, self.search_file.__name__, out=out)
            return False

        # Файл не передан
        if not path_to_file:
            self.message_error(self._file_name.format(ext.lower()), out=out)
            return False

        path_to_file = os.path.normpath(path_to_file)
        ext = ext.replace(".", "")

        # Передана директория
        if os.path.isdir(path_to_file) is True:
            self.message_error(self._dir_found, out=out)
            return False

        # Вывод сообщения
        self.message_info(self._file_find.format(self.message_line(os.path.basename(path_to_file))), out=out)

        self._file_load = self._file_find_hide  # Установка сообщения в исходное состояние

        _, extension = os.path.splitext(path_to_file)  # Расширение файла

        if ext != extension.replace(".", ""):
            self.message_error(self._wrong_extension.format(ext), space=self._space, out=out)
            return False

        # Файл не найден
        if os.path.isfile(path_to_file) is False:
            # Создание файла
            if create is True:
                open(path_to_file, "a", encoding="utf-8").close()

                self.message_info(
                    self._file_not_found_create.format(os.path.basename(path_to_file)), space=self._space, out=out
                )
                return False

            self.message_error(self._file_not_found.format(os.path.basename(path_to_file)), space=self._space, out=out)
            return False

        return True  # Результат

    def search_files(
        self, path_to_folder: str, exts: List[str], sort: bool = True, out: bool = True
    ) -> List[Optional[str]]:
        """Поиск файлов в указанной директории

        Args:
            path_to_folder (str): Путь к директории с файлами
            exts (List[str]): Расширения файлов
            sort (bool): Сортировать файлы
            out (bool): Печатать процесс выполнения

        Returns:
            List[Optional[str]]: список с найденными файлами
        """

        # Проверка аргументов
        if (
            type(path_to_folder) is not str
            or type(exts) is not list
            or len(exts) == 0
            or type(sort) is not bool
            or type(out) is not bool
        ):
            self.inv_args(__class__.__name__, self.search_files.__name__, out=out)
            return []

        path_to_folder = os.path.normpath(path_to_folder)
        exts = [ext.replace(".", "") for ext in exts]

        # Директория не передана
        if os.path.isdir(path_to_folder) is False:
            self.message_error(self._dir_name, out=out)
            return []

        # Вывод сообщения
        self.message_info(self._files_find.format(", ".join(x for x in exts), path_to_folder), out=out)

        # Список из файлов с необходимым расширением
        files = [str(p.resolve()) for p in Path(path_to_folder).glob("*") if p.suffix.replace(".", "") in exts]

        # В указанной директории не найдены необходимые файлы
        if len(files) == 0:
            self.message_error(self._files_not_found, space=self._space, out=out)
            return []

        # Сортировка файлов
        if sort is True:
            return sorted(files)

        return files

    def create_folder(self, path_to_folder: str, out: bool = True) -> bool:
        """Создание директории

        Args:
            path_to_folder (str): Путь к директории
            out (bool): Печатать процесс выполнения

        Returns:
            bool: **True** если директория создана, в обратном случае **False**
        """

        # Проверка аргументов
        if type(path_to_folder) is not str or not path_to_folder or type(out) is not bool:
            self.inv_args(__class__.__name__, self.create_folder.__name__, out=out)
            return False

        path_to_folder = os.path.normpath(path_to_folder)

        if not os.path.exists(path_to_folder):
            self.message_info(self._create_folder.format(path_to_folder), out=out)

            try:
                os.makedirs(path_to_folder)
            except Exception:
                self.message_error(self._folder_not_create.format(path_to_folder), space=4, out=out)
                return False
            else:
                return True
        else:
            return True

    def clear_folder(self, path_to_folder: str, out: bool = True) -> bool:
        """Очистка директории

        Args:
            path_to_folder (str): Путь к директории
            out (bool): Печатать процесс выполнения

        Returns:
            bool: **True** если директория очищена, в обратном случае **False**
        """

        # Проверка аргументов
        if type(path_to_folder) is not str or not path_to_folder or type(out) is not bool:
            self.inv_args(__class__.__name__, self.clear_folder.__name__, out=out)
            return False

        path_to_folder = os.path.normpath(path_to_folder)

        # Вывод сообщения
        self.message_info(self._clear_folder.format(path_to_folder), out=out)

        # Каталог с файлами найден
        if os.path.exists(path_to_folder):
            # Очистка
            for filename in os.listdir(path_to_folder):
                filepath = os.path.join(path_to_folder, filename)
                try:
                    shutil.rmtree(filepath)
                except OSError:
                    os.remove(filepath)
        else:
            self.message_error(self._clear_folder_not_found.format(path_to_folder), out=out)
            return False

        return True  # Результат



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Графический интерфейс пользователя для детектирования речевой активности в аудиовизуальном сигнале
"""

import os
import sys

PATH_TO_SOURCE = os.path.abspath(os.path.dirname(__file__))
PATH_TO_ROOT = os.path.join(PATH_TO_SOURCE, "..", "..")

sys.path.insert(0, os.path.abspath(PATH_TO_ROOT))

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Подавление Warning
import warnings

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

import importlib.resources as pkg_resources  # Работа с ресурсами внутри пакетов

from dataclasses import dataclass  # Класс данных

import logging  # Логирование

import streamlit as st  # Графический интерфейс пользователя
from PIL import Image

# Типы данных
from types import ModuleType

# Персональные
import openav  # Библиотека в целом
from openav.api.vad import RunVAD  # Сборка библиотеки

# Ресурсы библиотеки
from openav import rsrs
from openav.rsrs import favicon


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class MessagesVAD_GUI(RunVAD):
    """Класс для сообщений"""

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса


# ######################################################################################################################
# Выполняем только в том случае, если файл запущен сам по себе
# ######################################################################################################################
@dataclass
class RunVAD_GUI(MessagesVAD_GUI):
    """Класс для детектирования речевой активности в аудиовизуальном сигнале"""

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        #  Регистратор логирования с указанным именем
        self._logger_runvad_gui: logging.Logger = logging.getLogger(__class__.__name__)

        self.logger_gui = True  # Установка логирования GUI

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (защищенные)
    # ------------------------------------------------------------------------------------------------------------------

    # ------------------------------------------------------------------------------------------------------------------
    #  Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def run(self, metadata: ModuleType = openav, resources: ModuleType = rsrs, out: bool = True) -> bool:
        """Запуск детектирования речевой активности в аудиовизуальном сигнале

        Args:
            metadata (ModuleType): Модуль из которого необходимо извлечь информацию
            resources (ModuleType): Модуль с ресурсами
            out (bool): Печатать процесс выполнения

        Returns:
             bool: **True** если детектирование речевой активности в аудиовизуальном сигнале произведено успешно,
                   в обратном случае **False**
        """

        # Проверка аргументов
        if not isinstance(metadata, ModuleType) or not isinstance(resources, ModuleType) or type(out) is not bool:
            try:
                raise TypeError
            except Exception:
                self.inv_args(__class__.__name__, self.run.__name__, out=out)

                container = """<div style="
                    display:block; overflow:overlay; height:300px; padding:12px; border:1px solid #FFF
                ">"""

                for message in self._get_logger_messages_gui():
                    container += message

                container += "</div>"

                st.markdown(container, unsafe_allow_html=True)

                return False

        # # Ресурс с YAML файлом не найден
        # if pkg_resources.is_resource(module, path_to_file) is False:
        #     self.message_error(self._load_data_resources_not_found, space=self._space, out=out)
        #     return {}

        # # Открытие файла
        # with pkg_resources.open_text(module, path_to_file, encoding="utf-8", errors="strict") as yaml_data_file:
        #     try:
        #         config = yaml.load(yaml_data_file, Loader=yaml.FullLoader)
        #     except yaml.YAMLError:
        #         self.message_error(self._invalid_file, space=self._space, out=out)
        #         return {}

        # print(pkg_resources.is_resource(resources.favicon, "favicon.ico"))

        # with pkg_resources.open_text(
        #     resources, "/favicon/avicon.ico", encoding="utf-8", errors="strict"
        # ) as favicon_file:
        #     print(favicon_file.name)
        #     favicon_image = Image.open(favicon_file.name)

        #     # Настройки параметров страницы
        #     st.set_page_config(
        #         page_title=self._description,
        #         page_icon=favicon_image,
        #         layout="wide",
        #         initial_sidebar_state="expanded",
        #         menu_items={
        #             "About": self._description,
        #         },
        #     )


def main():
    # Запуск детектирования речевой активности в аудиовизуальном сигнале
    vad = RunVAD_GUI(lang="ru", path_to_logs="./openav/logs")
    vad.run(metadata=3, out=False)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class CustomException(Exception):
    """Класс для всех пользовательских исключений"""

    pass


class TypeEncodeVideoError(CustomException):
    """Указан неподдерживаемый тип кодирования видео"""

    pass


class PresetCFREncodeVideoError(CustomException):
    """Указан неподдерживаемый параметр обеспечивающий определенную скорость кодирования и сжатия видео"""

    pass


class TypeMessagesError(CustomException):
    """Указан неподдерживаемый тип сообщения"""

    pass


class SRInputTypeError(CustomException):
    """Указан неподдерживаемый тип файла для распознавания речи"""

    pass


class SamplingRateError(CustomException):
    """Указана неподдерживаемая частота дискретизации речевого сигнала"""

    pass


class WindowSizeSamplesError(CustomException):
    """Указано неподдерживаемое количество выборок в каждом окне"""

    pass


class IsNestedCatalogsNotFoundError(CustomException):
    """Вложенные директории, где хранятся данные не найдены"""

    pass


class IsNestedDirectoryVNotFoundError(CustomException):
    """Вложенная директория, для видеофрагментов не найдена"""

    pass


class IsNestedDirectoryANotFoundError(CustomException):
    """Вложенная директория, для аудиофрагментов не найдена"""

    pass


class InvalidContentLength(CustomException):
    """Не определен размер файла для загрузки"""

    pass


class CropPXError(CustomException):
    """Указан неверный диапазон обрезки в пикселях"""

    pass


class CropPercentsError(CustomException):
    """Указан неверный диапазон обрезки в процентах"""

    pass


class FlipLRProbabilityError(CustomException):
    """Указано неверное значение вероятности отражения по вертикальной оси"""

    pass


class FlipUDProbabilityError(CustomException):
    """Указано неверное значение вероятности отражения по горизонтальной оси"""

    pass


class BlurError(CustomException):
    """Указан неверный диапазон значений размытия"""

    pass


class ScaleError(CustomException):
    """Указан неверный диапазон значений масштабирования"""

    pass


class RotateError(CustomException):
    """Указан неверный диапазон значений угла наклона"""

    pass


class ContrastError(CustomException):
    """Указан неверный диапазон значений контрастности"""

    pass


class MixUpAlphaError(CustomException):
    """Указан неверный коэффициент для MixUp-аугментации"""

    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Определение языка
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Интернационализация (I18N) и локализация (L10N) (см. https://www.loc.gov/standards/iso639-2/php/code_list.php)
#     - brew install gettext (Если не установлен)
#     - brew link gettext --force
#     1. gettext --help
#     2. locate pygettext.py
#     3. /usr/local/Cellar/python@3.9/3.9.16/Frameworks/Python.framework/Versions/3.9/share/doc/python3.9/examples/
#        Tools/i18n/pygettext.py -d openav -o openav/modules/locales/base.pot openav
#     4. msgfmt --help
#     5. locate msgfmt.py
#     6. /usr/local/Cellar/python@3.9/3.9.16/Frameworks/Python.framework/Versions/3.9/share/doc/python3.9/examples/
#        Tools/i18n/msgfmt.py openav/modules/locales/en/LC_MESSAGES/base.po openav/modules/locales/en/LC_MESSAGES/base

# Подавление Warning
import warnings

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

from dataclasses import dataclass, field  # Класс данных

import os  # Взаимодействие с файловой системой
import sys  # Доступ к некоторым переменным и функциям Python
import gettext  # Формирование языковых пакетов
import inspect  # Инспектор
import argparse  # Парсинг аргументов и параметров командной строки

# Типы данных
from typing import List, Dict, Optional
from types import MethodType

# Персональные
from openav.modules.core.logging import Logging  # Логирование

# ######################################################################################################################
# Константы
# ######################################################################################################################
LANG: str = "ru"  # Язык


# ######################################################################################################################
# Интернационализация (I18N) и локализация (L10N)
# ######################################################################################################################
@dataclass
class Language(Logging):
    """Класс для интернационализации (I18N) и локализации (L10N)

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Язык
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    lang: str
    __lang: str = field(default=LANG, init=False, repr=False)
    """Язык

    .. note::
        private (приватный аргумент)
    """

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self.__i18n: List[Dict[str, MethodType]] = self.__get_locales()  # Получение языковых пакетов
        self._: MethodType = self.__set_locale(self.lang)  # Установка языка

        argparse._ = self.__i18n[1][self.lang]  # Установка языка для парсинга аргументов и параметров командной строки

    # ------------------------------------------------------------------------------------------------------------------
    # Свойства
    # ------------------------------------------------------------------------------------------------------------------

    @property
    def lang(self) -> str:
        """Получение/установка текущего языка

        Args:
            (str): Язык, доступные варианты:

                * ``"ru"`` - Русский язык (``по умолчанию``)
                * ``"en"`` - Английский язык

        Returns:
            str: Язык
        """

        return self.__lang

    @lang.setter
    def lang(self, lang: str):
        """Установка текущего языка"""

        try:
            # Проверка аргументов
            if type(lang) is not str or not lang or (lang in self.locales) is False:
                raise TypeError
        except TypeError:
            pass
        else:
            self.__lang = lang

    @property
    def locales(self) -> List[str]:
        """Получение поддерживаемых языков

        Returns:
            List[str]: Список поддерживаемых языков
        """

        return self.__get_languages()  # Поддерживаемые языки

    @property
    def path_to_locales(self) -> str:
        """Получение директории с языковыми пакетами

        Returns:
            str: Директория с языковыми пакетами
        """

        # Нормализация пути
        return os.path.normpath(os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "locales")))

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (приватные)
    # ------------------------------------------------------------------------------------------------------------------

    def __get_languages(self) -> List[Optional[str]]:
        """Получение поддерживаемых языков

        .. note::
            private (приватный метод)

        Returns:
            List[Optional[str]]: Список поддерживаемых языков
        """

        # Директория с языками найдена
        if os.path.exists(self.path_to_locales):
            # Формирование списка с подерживаемыми языками
            return next(os.walk(self.path_to_locales))[1]

        return []

    def __get_locales(self) -> List[Dict[str, MethodType]]:
        """Получение языковых пакетов

        .. note::
            private (приватный метод)

        Returns:
            List[Dict[str, MethodType]]: Список словарей с языковыми пакетами
        """

        # Языки
        tr_argparse = {}
        trs_base = {}

        # Проход по всем языкам
        for curr_lang in self.locales:
            tr_argparse[curr_lang] = gettext.translation(
                "argparse",  # Домен
                localedir=self.path_to_locales,  # Директория с поддерживаемыми языками
                languages=[curr_lang],  # Язык
                fallback=True,  # Отключение ошибки
            ).gettext

            trs_base[curr_lang] = gettext.translation(
                "base",  # Домен
                localedir=self.path_to_locales,  # Директория с поддерживаемыми языками
                languages=[curr_lang],  # Язык
                fallback=True,  # Отключение ошибки
            ).gettext

        return [trs_base, tr_argparse]

    def __set_locale(self, lang: str = LANG) -> MethodType:
        """Установка языка

        .. note::
            private (приватный метод)

        Args:
            lang (str): Язык

        Returns:
            MethodType: MethodType перевода строк на один из поддерживаемых языков если метод запущен через конструктор
        """

        try:
            # Проверка аргументов
            if type(lang) is not str:
                raise TypeError
        except TypeError:
            pass
        else:
            # Проход по всем поддерживаемым языкам
            for curr_lang in self.locales:
                # В аргументах командной строки не найден язык
                if curr_lang not in sys.argv:
                    # В аргументах метода не найден язык
                    if lang != curr_lang:
                        continue

                self.lang = curr_lang  # Изменение языка

            # Метод запущен в конструкторе
            if inspect.stack()[1].function == "__init__" or inspect.stack()[1].function == "__post_init__":
                return self.__i18n[0][self.lang]
            else:
                self._ = self.__i18n[0][self.lang]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Видеомодальность
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
import warnings
import logging
import absl.logging
import sys
import os  # Взаимодействие с файловой системой
import math

# Настройте фильтрацию предупреждений до импорта mediapipe
warnings.filterwarnings("ignore", category=UserWarning)
warnings.filterwarnings("ignore", category=FutureWarning)

# Установите уровень логирования
logging.getLogger().setLevel(logging.ERROR)

# Установите уровень логирования absl
absl.logging.set_verbosity(absl.logging.ERROR)

from dataclasses import dataclass  # Класс данных

import numpy as np  # Научные вычисления
import re  # Регулярные выражения
import filetype  # Определение типа файла и типа MIME
from PIL import Image  # Считывание изображений

with open(os.devnull, "w") as devnull:
    sys.stdout = devnull
    sys.stderr = devnull

    import mediapipe as mp
    import cv2

mp.solutions.face_mesh.FaceMesh()

# Типы данных
from typing import List, Set, Optional
from types import ModuleType

from pathlib import Path  # Работа с путями в файловой системе

# Персональные
from openav.modules.core.exceptions import (
    IsNestedCatalogsNotFoundError,
)
from openav.modules.file_manager.json_manager import Json  # Класс для работы с Json

sys.stdout = sys.__stdout__
sys.stderr = sys.__stderr__

# ######################################################################################################################
# Константы
# ######################################################################################################################

# Метрики оценки нейросетевой модели
METRICS_VIDEO: List[str] = [
    "accuracy",
]
DPI: List[int] = [72, 96, 150, 300, 600, 1200]  # DPI
COLOR_MODE: List[str] = ["gray", "rgb"]  # Цветовая гамма конечного изображения
RESIZE_RESAMPLE_MODE: List[str] = ["nearest", "bilinear", "lanczos"]  # Фильтры для масштабирования
EXT_VIDEO: List[str] = ["mov", "mp4", "webm"]  # Расширения искомых файлов
EXT_LIP: str = "png"
EXT_NPY: str = "npy"  # Расширения для сохранения сырых данных


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class VideoMessages(Json):
    """Класс для сообщений

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._subfolders_search: str = (
            self._('Поиск вложенных директорий в директории "{}" (глубина вложенности: {})') + self._em
        )
        self._subfolders_not_found: str = self._("В указанной директории вложенные директории не найдены") + self._em
        self._files_av_find: str = (
            self._('Поиск файлов с расширениями "{}" в директории "{}" (глубина вложенности: {})') + self._em
        )

        self.preprocess_video_files: str = self._("Предобработка речевых видеоданных") + self._em

        self._preprocess_true: str = self._("Все файлы успешно предобработаны") + self._em


# ######################################################################################################################
# Видео
# ######################################################################################################################
@dataclass
class Video(VideoMessages):
    """Класс для обработки видеомодальности

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        # ----------------------- Только для внутреннего использования внутри класса

        self.__dataset_preprocess_video: List[str] = []  # Пути до директорий с изображениями губ
        self.__unprocessed_files: List[str] = []  # Пути к файлам из которых области губ не извлечены
        self.__not_saved_files: List[str] = []  # Пути к файлам которые не сохранились при обработке

        self.__mp_face_mesh: Optional[ModuleType] = None

        self.__mp_drawing: Optional[ModuleType] = None
        self.__drawing_spec: Optional[mp.solutions.drawing_utils.DrawingSpec] = None

        self._lip_coords: Set[int] = set(
            [
                61,
                146,
                91,
                181,
                84,
                17,
                314,
                405,
                321,
                375,
                291,
                185,
                40,
                39,
                37,
                0,
                267,
                269,
                270,
                409,
                291,
                78,
                95,
                88,
                178,
                87,
                14,
                317,
                402,
                318,
                324,
                308,
                191,
                80,
                81,
                82,
                13,
                312,
                311,
                310,
                415,
                308,
            ]
        )

        self.__min_max_coords: List[int] = [-1, -1, -1, -1]  # min x, max x, min y, max y

        self._area_lip: List[int] = []
        self._area_lip_original: int = 0
        self._cnt_lip: int = 1

    # ------------------------------------------------------------------------------------------------------------------
    # Свойства
    # ------------------------------------------------------------------------------------------------------------------

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (приватные)
    # ------------------------------------------------------------------------------------------------------------------

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (защищенные)
    # ------------------------------------------------------------------------------------------------------------------

    # ------------------------------------------------------------------------------------------------------------------
    # Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def preprocess_video(
        self,
        depth: int = 1,
        resize: bool = True,
        resize_resample: str = "nearest",
        size_width: int = 112,
        size_height: int = 112,
        color_mode: str = "rgb",
        dpi: int = 1200,
        save_raw_data: bool = True,
        clear_dir_video: bool = False,
        out: bool = True,
    ) -> bool:
        """Предобработка речевых видеоданных

        Args:
            depth (int): Глубина иерархии для получения данных
            resize (bool):  Изменение размера кадра с найденной областью губ
            resize_resample (str): Фильтр для масштабирования
            size_width (int): Ширина области губ
            size_height (int): Высота области губ
            color_mode (str):  Цветовая гамма
            dpi (int): DPI
            save_raw_data (bool): Сохранение сырых данных с областями губ в формате .npy
            clear_dir_video (bool): Очистка директории для сохранения видео данных после предобработки
            out (bool) Отображение

        Returns:
            bool: **True** если предобработка речевых видеоданных произведена, в обратном случае
            **False**
        """

        try:
            # Проверка аргументов
            if (
                type(depth) is not int
                or depth < 1
                or type(resize) is not bool
                or type(size_width) is not int
                or type(size_height) is not int
                or type(color_mode) is not str
                or (color_mode in COLOR_MODE) is False
                or type(dpi) is not int
                or (dpi in DPI) is False
                or type(save_raw_data) is not bool
                or type(clear_dir_video) is not bool
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.preprocess_video.__name__, out=out)
            return False
        else:
            # Информационное сообщение
            self.message_info(
                self._subfolders_search.format(
                    self.message_line(self.path_to_dataset),
                    self.message_line(str(depth)),
                ),
                out=out,
            )

            # Создание директории, где хранятся данные
            if self.create_folder(self.path_to_dataset, out=False) is False:
                return False

            # Получение вложенных директорий, где хранятся данные
            nested_paths = self.get_paths(self.path_to_dataset, depth=depth, out=False)

            # Вложенные директории не найдены
            try:
                if len(nested_paths) == 0:
                    raise IsNestedCatalogsNotFoundError
            except IsNestedCatalogsNotFoundError:
                self.message_error(self._subfolders_not_found, space=self._space, out=out)
                return False

            # Информационное сообщение
            self.message_info(
                self._files_av_find.format(
                    self.message_line(", ".join(x.replace(".", "") for x in self.ext_search_files)),
                    self.message_line(self.path_to_dataset),
                    self.message_line(str(depth)),
                ),
                out=out,
            )

            paths = []  # Пути до визуальных файлов

            # Проход по всем вложенным директориям
            for nested_path in nested_paths:
                # Формирование списка с видеофайлами
                for p in Path(nested_path).glob("*"):
                    # Добавление текущего пути к видеофайлу в список
                    if p.suffix.lower() in self.ext_search_files:
                        paths.append(p.resolve())

            # Директория с набором данных не содержит визуальных файлов с необходимыми расширениями
            try:
                self.__len_paths = len(paths)  # Количество визуальных файлов

                if self.__len_paths == 0:
                    raise TypeError
            except TypeError:
                self.message_error(self._files_not_found, space=self._space, out=out)
                return False
            except Exception:
                self.message_error(self._unknown_err, space=self._space, out=out)
                return False
            else:
                # Очистка директории для сохранения фрагментов визуального сигнала
                if clear_dir_video is True and os.path.exists(self.path_to_dataset_video) is True:
                    if self.clear_folder(self.path_to_dataset_video, out=False) is False:
                        return False

                self.__dataset_preprocess_video = []  # Пути до директорий с изображениями губ

                self.__unprocessed_files = []  # Пути к файлам из которых области губ не извлечены

                # Информационное сообщение
                self.message_info(self.preprocess_video_files, out=out)

                # Локальный путь
                self.__local_path = lambda lp: os.path.join(
                    *Path(lp).parts[-abs((len(Path(lp).parts) - len(Path(self.path_to_dataset).parts))) :]
                )

                try:
                    self.__mp_face_mesh = mp.solutions.face_mesh

                    self.__mp_drawing = mp.solutions.drawing_utils
                    self.__drawing_spec = self.__mp_drawing.DrawingSpec(thickness=1, circle_radius=1)
                except Exception:
                    self.message_error(self._unknown_err, space=self._space, out=out)
                    return False
                else:
                    # Проход по всем найденным визуальных файлам
                    for i, path in enumerate(paths):
                        self._cnt_lip = 1

                        self.__curr_path = path  # Текущий визуальный файл
                        self.__i = i + 1  # Счетчик

                        self.message_progressbar(
                            self._curr_progress.format(
                                self.__i,
                                self.__len_paths,
                                round(self.__i * 100 / self.__len_paths, 2),
                                self.message_line(self.__local_path(self.__curr_path)),
                            ),
                            space=self._space,
                            out=out,
                        )

                        self.__splitted_path = str(
                            self.__curr_path.parent.relative_to(Path(self.path_to_dataset))
                        ).strip()

                        self.__curr_path = str(self.__curr_path)

                        # Пропуск невалидных значений
                        if not self.__splitted_path or re.search(r"\s", self.__splitted_path) is not None:
                            continue

                        # Тип файла
                        kind = filetype.guess(self.__curr_path)

                        try:
                            # Видео
                            if kind.mime.startswith("video/") is True:
                                cap = cv2.VideoCapture(self.__curr_path)

                                if not os.path.exists(self.path_to_dataset_video):
                                    # Директория не создана
                                    if self.create_folder(self.path_to_dataset_video, out=False) is False:
                                        raise FileNotFoundError

                                path_to_subfolder = os.path.join(
                                    self.path_to_dataset_video, Path(self.__curr_path).stem
                                )

                                # Очистка директории
                                if clear_dir_video is True and os.path.exists(path_to_subfolder) is True:
                                    if (
                                        self.clear_folder(
                                            path_to_subfolder,
                                            out=False,
                                        )
                                        is False
                                    ):
                                        return False

                                if not os.path.exists(path_to_subfolder):
                                    # Директория не создана
                                    if (
                                        self.create_folder(
                                            path_to_subfolder,
                                            out=False,
                                        )
                                        is False
                                    ):
                                        raise FileNotFoundError

                                with self.__mp_face_mesh.FaceMesh(
                                    static_image_mode=False,
                                    max_num_faces=1,
                                    min_detection_confidence=0.5,
                                ) as face_mesh:
                                    while cap.isOpened():
                                        _, curr_frame = cap.read()

                                        if curr_frame is None:
                                            break

                                        results = face_mesh.process(cv2.cvtColor(curr_frame, cv2.COLOR_BGR2RGB))

                                        if results.multi_face_landmarks:
                                            frame_height, frame_width, _ = curr_frame.shape

                                            for face_landmarks in results.multi_face_landmarks:
                                                land_x = []
                                                land_y = []

                                                for idx5, landmark in enumerate(face_landmarks.landmark):
                                                    x = min(math.floor(landmark.x * frame_width), frame_width - 1)
                                                    y = min(math.floor(landmark.y * frame_height), frame_height - 1)

                                                    if idx5 in self._lip_coords:
                                                        land_x.append(x)
                                                        land_y.append(y)

                                                self.__min_max_coords[0] = min(land_x)
                                                self.__min_max_coords[1] = max(land_x)
                                                self.__min_max_coords[2] = min(land_y)
                                                self.__min_max_coords[3] = max(land_y)

                                                area = (self.__min_max_coords[1] - self.__min_max_coords[0]) * (
                                                    self.__min_max_coords[3] - self.__min_max_coords[2]
                                                )

                                                if self._cnt_lip == 0:
                                                    self._area_lip_original = area

                                                self._area_lip.append(area)

                                                lip_roi = curr_frame[
                                                    self.__min_max_coords[2] : self.__min_max_coords[3],
                                                    self.__min_max_coords[0] : self.__min_max_coords[1],
                                                    :,
                                                ]

                                                lip_roi_path = os.path.join(
                                                    path_to_subfolder, str(self._cnt_lip) + "." + EXT_LIP
                                                )

                                                lip_roi = lip_roi[:, :, ::-1]

                                                # Создание и сохранение изображения с помощью Pillow
                                                img = Image.fromarray(lip_roi)

                                                if color_mode == COLOR_MODE[0]:
                                                    img = img.convert("L")

                                                if resize is True:
                                                    if resize_resample is RESIZE_RESAMPLE_MODE[0]:
                                                        resize_resample = Image.NEAREST
                                                    elif resize_resample is RESIZE_RESAMPLE_MODE[1]:
                                                        resize_resample = Image.BILINEAR
                                                    elif resize_resample is RESIZE_RESAMPLE_MODE[2]:
                                                        resize_resample = Image.LANCZOS
                                                    else:
                                                        resize_resample = Image.NEAREST

                                                    img = img.resize(
                                                        (size_width, size_height), resample=resize_resample
                                                    )

                                                img.save(lip_roi_path, dpi=(dpi, dpi))

                                                if save_raw_data:
                                                    # Сохранение сырых данных в формате .npy
                                                    raw_data_path = lip_roi_path.replace("." + EXT_LIP, "." + EXT_NPY)
                                                    np.save(raw_data_path, lip_roi)
                                        else:
                                            self.__min_max_coords = [-1, -1, -1, -1]

                                        self._cnt_lip += 1
                        except Exception:
                            self.__unprocessed_files.append(self.__curr_path)
                            self.message_progressbar(close=True, out=out)
                            continue

                    self.message_progressbar(close=True, out=out)

                    # Файлы на которых предварительная обработка не отработала
                    unprocessed_files_unique = np.unique(np.array(self.__unprocessed_files)).tolist()

                    if len(unprocessed_files_unique) == 0 and len(self.__not_saved_files) == 0:
                        self.message_true(self._preprocess_true, space=self._space, out=out)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Формирование набора данных
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################

import os
import subprocess
from torch.utils.data import Dataset
import torchaudio
import mediapipe as mp
import math
import numpy as np
import cv2
import librosa
import torch
from einops import rearrange

# ######################################################################################################################
# Константы
# ######################################################################################################################


# ######################################################################################################################
# Сообщения
# ######################################################################################################################


# ######################################################################################################################
# Набор данных
# ######################################################################################################################


class AVDataset(Dataset):
    def __init__(
        self,
        path_files=[],
        labels=[],
        subset="",
        shape=88,
        sampling_rate=16000,
        len_audio=19495,
        len_video=29,
        max_segment=1,
    ):
        self.path_files = path_files  # пути к видео
        self.labels = labels  # метки
        self.subset = subset  # имя выборки для сохранения данных
        self.shape_frame = shape  # фиксированный размер кадра
        self.sampling_rate = sampling_rate  # частота дискретизаци
        self.len_audio = len_audio  # длина аудио сегмента в отчетах
        self.len_video = len_video  # длина видео сегмента в кадрах
        self.max_segment = max_segment  # количество аудио и видео сегментов

        self.face_detector = mp.solutions.face_mesh  # инициализация детектора лиц

        self.lips = [
            61,
            146,
            91,
            181,
            84,
            17,
            314,
            405,
            321,
            375,
            291,
            409,
            270,
            269,
            267,
            0,
            37,
            39,
            40,
            185,
        ]  # координаты губ

        self.pl = [2048, 64, None, True, "reflect", 2.0, 64, "slaney", True, None]

    def norm_coordinates(self, normalized_x, normalized_y, image_width, image_height):

        x_px = min(math.floor(normalized_x * image_width), image_width - 1)
        y_px = min(math.floor(normalized_y * image_height), image_height - 1)

        return x_px, y_px

    def pad_sequence(self, lips):
        current_length = len(lips)

        if current_length < self.len_video:
            lips.extend([lips[-1]] * (self.len_video - current_length))

        return lips

    def get_box(self, fl, w, h):
        idx_to_coors = {}
        for idx, landmark in enumerate(fl.landmark):
            landmark_px = self.norm_coordinates(landmark.x, landmark.y, w, h)
            if landmark_px:
                idx_to_coors[idx] = landmark_px

        ys = [v[1] for _, v in idx_to_coors.items()]
        xs = [v[0] for _, v in idx_to_coors.items()]

        x_min, y_min = min(xs), min(ys)
        x_max, y_max = max(xs), max(ys)

        (x_min, y_min) = max(x_min, 0), max(y_min, 0)
        (x_max, y_max) = min(x_max, w - 1), min(y_max, h - 1)

        return x_min, y_min, x_max, y_max

    def convert_video_to_audio(self, path, sampling_rate=16000):
        path_save = path

        if path[-3:] != "wav":
            path_save = path[:-3] + "wav"
            if not os.path.exists(path_save):
                ff_audio = "ffmpeg -i {} -vn -acodec pcm_s16le -ar 44100 -ac 2 {}".format(path, path_save)
                subprocess.call(ff_audio, shell=True)

        wav, sr = torchaudio.load(path_save)
        if wav.size(0) > 1:
            wav = wav.mean(dim=0, keepdim=True)

        if sr != sampling_rate:
            transform = torchaudio.transforms.Resample(orig_freq=sr, new_freq=sampling_rate)
            wav = transform(wav)
            sr = sampling_rate

        assert sr == sampling_rate
        return wav.squeeze(0)

    def make_padding(self, img):
        if img.shape[0] > 0 and img.shape[1] > 0:
            factor_0 = self.shape_frame / img.shape[0]
            factor_1 = self.shape_frame / img.shape[1]
            factor = min(factor_0, factor_1)
            dsize = (int(img.shape[1] * factor), int(img.shape[0] * factor))
            img = cv2.resize(img, dsize)
            diff_0 = self.shape_frame - img.shape[0]
            diff_1 = self.shape_frame - img.shape[1]
            img = np.pad(
                img, ((diff_0 // 2, diff_0 - diff_0 // 2), (diff_1 // 2, diff_1 - diff_1 // 2), (0, 0)), "mean"
            )
        if img.shape[0:2] != (self.shape_frame, self.shape_frame):
            img = cv2.resize(img, (self.shape_frame, self.shape_frame))
        return img

    def get_metadata(self, name_file):
        a_fss = []
        v_fss = []
        lip_areas = []

        video = cv2.VideoCapture(name_file)

        len_audio = self.len_audio

        w = int(video.get(cv2.CAP_PROP_FRAME_WIDTH))
        h = int(video.get(cv2.CAP_PROP_FRAME_HEIGHT))
        total_frames = int(video.get(cv2.CAP_PROP_FRAME_COUNT))

        with self.face_detector.FaceMesh(
            max_num_faces=1, refine_landmarks=False, min_detection_confidence=0.5, min_tracking_confidence=0.5
        ) as face_mesh:

            while video.isOpened():
                success, image = video.read()

                if image is None:
                    break

                image.flags.writeable = False
                image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
                results = face_mesh.process(image)
                image.flags.writeable = True

                image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
                if results.multi_face_landmarks:
                    for id_face, fl in enumerate(results.multi_face_landmarks):
                        x_norm_list = []
                        y_norm_list = []
                        for xxxx in self.lips:
                            x_px, y_px = self.norm_coordinates(fl.landmark[xxxx].x, fl.landmark[xxxx].y, w, h)
                            x_norm_list.append(x_px)
                            y_norm_list.append(y_px)
                        startX = int(min(x_norm_list))
                        startY = int(min(y_norm_list))
                        endX = int(max(x_norm_list))
                        endY = int(max(y_norm_list))
                        (startX, startY) = (max(0, startX), max(0, startY))
                        (endX, endY) = (min(w - 1, endX), min(h - 1, endY))
                        cur_fr = image[startY:endY, startX:endX]
                        cur_fr = self.make_padding(cur_fr)
                        lip_areas.append(cur_fr / 255.0)
                else:
                    lip_areas.append(np.zeros((self.shape_frame, self.shape_frame, 3)))

        for i in range(0, total_frames, self.len_video):
            curr_frames = lip_areas[i : min(i + self.len_video, total_frames)]
            curr_frames = self.pad_sequence(curr_frames)
            v_fss.append(curr_frames)
            if i + self.len_video >= total_frames:
                break

        len_spec = len_audio / self.pl[1]
        if math.modf(len_spec)[0] == 0:
            len_spec += 1
        len_spec = math.ceil(len_spec) + 1

        try:
            wav = self.convert_video_to_audio(name_file)
            for i in range(0, len(wav) + 1, len_audio):
                curr_wav = wav[i : min(i + len_audio, len(wav))]
                if len(curr_wav) > self.pl[0]:
                    m_s = librosa.feature.melspectrogram(
                        y=np.asarray(curr_wav),
                        sr=self.sampling_rate,
                        n_fft=self.pl[0],
                        hop_length=self.pl[1],
                        win_length=self.pl[2],
                        center=self.pl[3],
                        pad_mode=self.pl[4],
                        power=self.pl[5],
                        n_mels=self.pl[6],
                        norm=self.pl[7],
                        htk=self.pl[8],
                        fmax=self.pl[9],
                    )

                    db_m_s = librosa.power_to_db(m_s, top_db=80)

                    if db_m_s.shape[1] < len_spec:
                        db_m_s = np.pad(db_m_s, ((0, 0), (0, len_spec - db_m_s.shape[1])), "mean")

                    db_m_s = db_m_s / 255.0 / 255.0
                    db_m_s = np.expand_dims(db_m_s, axis=-1)
                    a_fss.append(db_m_s)

                if i + len_audio >= len(wav):
                    break
        except Exception:
            pass

        if len(a_fss) > self.max_segment:
            a_fss = a_fss[: self.max_segment]
        elif len(a_fss) < self.max_segment:
            a_fss.extend([a_fss[-1]] * (self.max_segment - len(a_fss)))

        if len(v_fss) > self.max_segment:
            v_fss = v_fss[: self.max_segment]
        elif len(v_fss) < self.max_segment:
            v_fss.extend([v_fss[-1]] * (self.max_segment - len(v_fss)))

        return np.array(a_fss), np.array(v_fss)

    def __getitem__(self, index):
        a_fss, v_fss = self.get_metadata(str(self.path_files[index]))
        label = self.labels[index]  # метки
        return torch.FloatTensor(a_fss), torch.FloatTensor(v_fss), label

    def __len__(self):
        return len(self.path_files)


class AVTest:
    def __init__(
        self,
        shape=88,
        sampling_rate=16000,
        len_audio=19495,
        len_video=29,
        max_segment=1,
    ):
        self.shape_frame = shape  # фиксированный размер кадра
        self.sampling_rate = sampling_rate  # частота дискретизаци
        self.len_audio = len_audio  # длина аудио сегмента в отчетах
        self.len_video = len_video  # длина видео сегмента в кадрах
        self.max_segment = max_segment  # количество аудио и видео сегментов

        self.face_detector = mp.solutions.face_mesh  # инициализация детектора лиц

        self.lips = [
            61,
            146,
            91,
            181,
            84,
            17,
            314,
            405,
            321,
            375,
            291,
            409,
            270,
            269,
            267,
            0,
            37,
            39,
            40,
            185,
        ]  # координаты губ

        self.pl = [2048, 64, None, True, "reflect", 2.0, 64, "slaney", True, None]

    def norm_coordinates(self, normalized_x, normalized_y, image_width, image_height):

        x_px = min(math.floor(normalized_x * image_width), image_width - 1)
        y_px = min(math.floor(normalized_y * image_height), image_height - 1)

        return x_px, y_px

    def pad_sequence(self, lips):
        current_length = len(lips)

        if current_length < self.len_video:
            lips.extend([lips[-1]] * (self.len_video - current_length))

        return lips

    def get_box(self, fl, w, h):
        idx_to_coors = {}
        for idx, landmark in enumerate(fl.landmark):
            landmark_px = self.norm_coordinates(landmark.x, landmark.y, w, h)
            if landmark_px:
                idx_to_coors[idx] = landmark_px

        ys = [v[1] for _, v in idx_to_coors.items()]
        xs = [v[0] for _, v in idx_to_coors.items()]

        x_min, y_min = min(xs), min(ys)
        x_max, y_max = max(xs), max(ys)

        (x_min, y_min) = max(x_min, 0), max(y_min, 0)
        (x_max, y_max) = min(x_max, w - 1), min(y_max, h - 1)

        return x_min, y_min, x_max, y_max

    def convert_video_to_audio(self, path, sampling_rate=16000):
        path_save = path

        if path[-3:] != "wav":
            path_save = path[:-3] + "wav"
            if not os.path.exists(path_save):
                ff_audio = "ffmpeg -i {} -vn -acodec pcm_s16le -ar 44100 -ac 2 {}".format(path, path_save)
                subprocess.call(ff_audio, shell=True)

        wav, sr = torchaudio.load(path_save)
        if wav.size(0) > 1:
            wav = wav.mean(dim=0, keepdim=True)

        if sr != sampling_rate:
            transform = torchaudio.transforms.Resample(orig_freq=sr, new_freq=sampling_rate)
            wav = transform(wav)
            sr = sampling_rate

        assert sr == sampling_rate
        return wav.squeeze(0)

    def make_padding(self, img):
        if img.shape[0] > 0 and img.shape[1] > 0:
            factor_0 = self.shape_frame / img.shape[0]
            factor_1 = self.shape_frame / img.shape[1]
            factor = min(factor_0, factor_1)
            dsize = (int(img.shape[1] * factor), int(img.shape[0] * factor))
            img = cv2.resize(img, dsize)
            diff_0 = self.shape_frame - img.shape[0]
            diff_1 = self.shape_frame - img.shape[1]
            img = np.pad(
                img, ((diff_0 // 2, diff_0 - diff_0 // 2), (diff_1 // 2, diff_1 - diff_1 // 2), (0, 0)), "mean"
            )
        if img.shape[0:2] != (self.shape_frame, self.shape_frame):
            img = cv2.resize(img, (self.shape_frame, self.shape_frame))
        return img

    def get_metadata(self, name_file):
        a_fss = []
        v_fss = []
        lip_areas = []

        # работа с видео сигналом
        video = cv2.VideoCapture(name_file)

        len_audio = self.len_audio

        w = int(video.get(cv2.CAP_PROP_FRAME_WIDTH))
        h = int(video.get(cv2.CAP_PROP_FRAME_HEIGHT))
        total_frames = int(video.get(cv2.CAP_PROP_FRAME_COUNT))

        with self.face_detector.FaceMesh(
            max_num_faces=1, refine_landmarks=False, min_detection_confidence=0.5, min_tracking_confidence=0.5
        ) as face_mesh:

            while video.isOpened():
                success, image = video.read()

                if image is None:
                    break

                image.flags.writeable = False
                image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
                results = face_mesh.process(image)
                image.flags.writeable = True

                image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
                if results.multi_face_landmarks:
                    for id_face, fl in enumerate(results.multi_face_landmarks):
                        x_norm_list = []
                        y_norm_list = []
                        for xxxx in self.lips:
                            x_px, y_px = self.norm_coordinates(fl.landmark[xxxx].x, fl.landmark[xxxx].y, w, h)
                            x_norm_list.append(x_px)
                            y_norm_list.append(y_px)
                        startX = int(min(x_norm_list))
                        startY = int(min(y_norm_list))
                        endX = int(max(x_norm_list))
                        endY = int(max(y_norm_list))
                        (startX, startY) = (max(0, startX), max(0, startY))
                        (endX, endY) = (min(w - 1, endX), min(h - 1, endY))
                        cur_fr = image[startY:endY, startX:endX]
                        cur_fr = self.make_padding(cur_fr)
                        lip_areas.append(cur_fr / 255.0)
                else:
                    lip_areas.append(np.zeros((self.shape_frame, self.shape_frame, 3)))

        for i in range(0, total_frames, self.len_video):
            curr_frames = lip_areas[i : min(i + self.len_video, total_frames)]
            curr_frames = self.pad_sequence(curr_frames)
            v_fss.append(curr_frames)
            if i + self.len_video >= total_frames:
                break

        len_spec = len_audio / self.pl[1]
        if math.modf(len_spec)[0] == 0:
            len_spec += 1
        len_spec = math.ceil(len_spec) + 1

        wav = self.convert_video_to_audio(name_file)
        for i in range(0, len(wav) + 1, len_audio):
            curr_wav = wav[i : min(i + len_audio, len(wav))]
            if len(curr_wav) > self.pl[0]:
                m_s = librosa.feature.melspectrogram(
                    y=np.asarray(curr_wav),
                    sr=self.sampling_rate,
                    n_fft=self.pl[0],
                    hop_length=self.pl[1],
                    win_length=self.pl[2],
                    center=self.pl[3],
                    pad_mode=self.pl[4],
                    power=self.pl[5],
                    n_mels=self.pl[6],
                    norm=self.pl[7],
                    htk=self.pl[8],
                    fmax=self.pl[9],
                )

                db_m_s = librosa.power_to_db(m_s, top_db=80)

                if db_m_s.shape[1] < len_spec:
                    db_m_s = np.pad(db_m_s, ((0, 0), (0, len_spec - db_m_s.shape[1])), "mean")

                db_m_s = db_m_s / 255.0 / 255.0
                db_m_s = np.expand_dims(db_m_s, axis=-1)
                a_fss.append(db_m_s)

            if i + len_audio >= len(wav):
                break

        if len(a_fss) > self.max_segment:
            a_fss = a_fss[: self.max_segment]
        elif len(a_fss) < self.max_segment:
            a_fss.extend([a_fss[-1]] * (self.max_segment - len(a_fss)))

        if len(v_fss) > self.max_segment:
            v_fss = v_fss[: self.max_segment]
        elif len(v_fss) < self.max_segment:
            v_fss.extend([v_fss[-1]] * (self.max_segment - len(v_fss)))

        audio_data = torch.FloatTensor(np.array(a_fss))
        video_data = torch.FloatTensor(np.array(v_fss))

        audio_data = audio_data.unsqueeze(0)
        audio_data = rearrange(audio_data, "b g n l c -> b g c n l")
        video_data = video_data.unsqueeze(0)
        video_data = rearrange(video_data, "b g1 g2 h w c -> b g1 g2 c h w")

        return name_file, audio_data, video_data

    def __getitem__(self, index):
        a_fss, v_fss = self.get_metadata(str(self.path_files[index]))
        label = self.labels[index]
        return torch.FloatTensor(a_fss), torch.FloatTensor(v_fss), label

    def __len__(self):
        return len(self.path_files)
